@@ -168,7 +168,7 @@ void logotest(void)
 }
 
 
-void main(void)
+int main(void)
 {
     vga_init();
     VGAMODE = vga_getdefaultmode();
