@@ -59,7 +59,7 @@ void by8test(void)
 }
 
 
-void main(int argc, char *argv[])
+int main(int argc, char *argv[])
 {
     int i,j;
 
@@ -118,4 +118,5 @@ void main(int argc, char *argv[])
       }
     getchar();
     vga_setmode(TEXT);
+    return 0;
 }
