@@ -17,6 +17,8 @@ extern "C"
 {
 #endif
 
+#define SVGALIB_VER 0x010900
+
 #define TEXT 	     0		/* Compatible with VGAlib v1.2 */
 #define G320x200x16  1
 #define G640x200x16  2
@@ -77,9 +79,127 @@ extern "C"
 #define G1600x1200x16M 48
 #define G1600x1200x16M32 49
 
-#define __GLASTMODE G1600x1200x16M32
+#define G320x240x256V 50
+#define G320x240x32K 51
+#define G320x240x64K 52
+#define G320x240x16M 53
+#define G320x240x16M32 54
+
+#define G400x300x256 55
+#define G400x300x32K 56
+#define G400x300x64K 57
+#define G400x300x16M 58
+#define G400x300x16M32 59
+
+#define G512x384x256 60
+#define G512x384x32K 61
+#define G512x384x64K 62
+#define G512x384x16M 63
+#define G512x384x16M32 64
+
+#define G960x720x256 65
+#define G960x720x32K 66
+#define G960x720x64K 67
+#define G960x720x16M 68
+#define G960x720x16M32 69
+
+#define G1920x1440x256 70
+#define G1920x1440x32K 71
+#define G1920x1440x64K 72
+#define G1920x1440x16M 73
+#define G1920x1440x16M32 74
+
+/* The following modes have been introduced by SciTech Display Doctor */
+
+#define G320x400x256V 75
+#define G320x400x32K 76
+#define G320x400x64K 77
+#define G320x400x16M 78
+#define G320x400x16M32 79
+
+#define G640x400x256 80
+#define G640x400x32K 81
+#define G640x400x64K 82
+#define G640x400x16M 83
+#define G640x400x16M32 84
+
+#define G320x480x256 85
+#define G320x480x32K 86
+#define G320x480x64K 87
+#define G320x480x16M 88
+#define G320x480x16M32 89
+
+#define G720x540x256 90
+#define G720x540x32K 91
+#define G720x540x64K 92
+#define G720x540x16M 93
+#define G720x540x16M32 94
+
+#define G848x480x256 95
+#define G848x480x32K 96
+#define G848x480x64K 97
+#define G848x480x16M 98
+#define G848x480x16M32 99
+
+#define G1072x600x256 100
+#define G1072x600x32K 101
+#define G1072x600x64K 102
+#define G1072x600x16M 103
+#define G1072x600x16M32 104
+
+#define G1280x720x256 105
+#define G1280x720x32K 106
+#define G1280x720x64K 107
+#define G1280x720x16M 108
+#define G1280x720x16M32 109
+
+#define G1360x768x256 110
+#define G1360x768x32K 111
+#define G1360x768x64K 112
+#define G1360x768x16M 113
+#define G1360x768x16M32 114
+
+#define G1800x1012x256 115
+#define G1800x1012x32K 116
+#define G1800x1012x64K 117
+#define G1800x1012x16M 118
+#define G1800x1012x16M32 119
+
+#define G1920x1080x256 120
+#define G1920x1080x32K 121
+#define G1920x1080x64K 122
+#define G1920x1080x16M 123
+#define G1920x1080x16M32 124
+
+#define G2048x1152x256 125
+#define G2048x1152x32K 126
+#define G2048x1152x64K 127
+#define G2048x1152x16M 128
+#define G2048x1152x16M32 129
+
+#define G2048x1536x256 130
+#define G2048x1536x32K 131
+#define G2048x1536x64K 132
+#define G2048x1536x16M 133
+#define G2048x1536x16M32 134
+
+#define G512x480x256 135
+#define G512x480x32K 136
+#define G512x480x64K 137
+#define G512x480x16M 138
+#define G512x480x16M32 139
+
+#define G400x600x256 140
+#define G400x600x32K 141
+#define G400x600x64K 142
+#define G400x600x16M 143
+#define G400x600x16M32 144
+
+#define __GLASTMODE G400x600x16M32
 #define GLASTMODE vga_lastmodenumber()
 
+    extern int vga_version;
+
     extern int vga_setmode(int mode);
     extern int vga_hasmode(int mode);
     extern int vga_setflipchar(int c);
@@ -167,6 +287,7 @@ extern "C"
     extern char *vga_getmodename(int mode);
     extern int vga_getmodenumber(char *name);
     extern int vga_lastmodenumber(void);
+    extern int vga_getoptmode(int x, int y, int colors, int bytesperpixel, int c);
 
     extern unsigned char *graph_mem;
     extern unsigned char *vga_getgraphmem(void);
@@ -197,6 +318,9 @@ extern "C"
     extern int vga_setlinearaddressing(void);
     extern void vga_safety_fork(void (*shutdown_routine) (void));
 
+    extern int vga_simple_init(void);
+    extern void vga_chipset_saveregs(unsigned char *);
+    extern void vga_chipset_setregs(unsigned char *);
 
 #ifdef EGA			/* Kernel headers may define this. */
 #undef EGA
@@ -222,6 +346,13 @@ extern "C"
 #define NV3		17
 #define ET6000		18
 #define VESA		19
+#define MX              20
+#define PARADISE	21
+#define RAGE		22
+#define BANSHEE		23
+#define SIS		24
+#define I740		25
+#define NEOMAGIC	26
 
     /* Hor. sync: */
 #define MON640_60	0	/* 31.5 KHz (standard VGA) */
@@ -251,6 +382,44 @@ extern "C"
     extern int vga_ext_set(unsigned what,...);
     extern int vga_accel(unsigned operation,...);
 
+    extern int vga_addtiming(int pixelClock,
+       			      int HDisplay,		
+                              int HSyncStart,
+                              int HSyncEnd,
+                              int HTotal,
+                              int VDisplay,
+                              int VSyncStart,
+                              int VSyncEnd,
+                              int VTotal,
+                              int flags);
+
+    extern int vga_changetiming(int pixelClock,
+       			      int HDisplay,		
+                              int HSyncStart,
+                              int HSyncEnd,
+                              int HTotal,
+                              int VDisplay,
+                              int VSyncStart,
+                              int VSyncEnd,
+                              int VTotal,
+                              int flags);
+
+   extern int vga_getcurrenttiming(int *pixelClock,
+       			      int *HDisplay,		
+                              int *HSyncStart,
+                              int *HSyncEnd,
+                              int *HTotal,
+                              int *VDisplay,
+                              int *VSyncStart,
+                              int *VSyncEnd,
+                              int *VTotal,
+                              int *flags);
+
+   extern int vga_addmode(int xdim, int ydim, int cols, 
+                          int xbytes, int bytespp);
+
+   extern int vga_guesstiming(int x, int y, int clue, int arg);
+
 /* Valid values for what in vga_ext_set: */
 #define VGA_EXT_AVAILABLE	0	/* supported flags */
 #define VGA_EXT_SET		1	/* set flag(s) */
@@ -365,7 +534,6 @@ extern void vga_runinbackground(int stat, ...);
 #define VGA_COMEFROMBACK -2
 extern int vga_runinbackground_version(void);
 
-
 #ifdef __cplusplus
 }
 
