@@ -1,5 +1,5 @@
 /*
-At3d (at25) driver - Matan Ziv-Av zivav@cs.bgu.ac.il
+At3d (at25) driver - Matan Ziv-Av, matan@svgalib.org
 please report problems to me,
 
 If you have an alliance promotion 6422, it should
@@ -24,21 +24,22 @@ there are still the following problems:
 #include <stdio.h>		/* for printf */
 #include <string.h>		/* for memset */
 #include <unistd.h>
+#include <sys/mman.h>
 #include "vga.h"
 #include "libvga.h"
 #include "driver.h"
-
+#include "vgapci.h"
 
 /* New style driver interface. */
 #include "timing.h"
 #include "vgaregs.h"
+#include "vgammvgaio.h"
 #include "interface.h"
 #include "accel.h"
 
 #define APMREG_SAVE(i) (VGA_TOTAL_REGS+i)
 #define APM_TOTAL_REGS (VGA_TOTAL_REGS + 38)
 
-
 static int apm_init(int, int, int);
 static void apm_unlock(void);
 
@@ -51,25 +52,25 @@ static CardSpecs *cardspecs;
 
 static void outXR(int index,unsigned char d)
 {
-    __svgalib_outbSR(0x1d,index>>2);
+    __svgalib_outseq(0x1d,index>>2);
     outb(apm_xbase,d);
 }
 
 static void outwXR(int index,unsigned short d)
 {
-    __svgalib_outbSR(0x1d,index>>2);
+    __svgalib_outseq(0x1d,index>>2);
     outw(apm_xbase,d);
 }
 
 static void outlXR(int index,unsigned long d)
 {
-    __svgalib_outbSR(0x1d,index>>2);
+    __svgalib_outseq(0x1d,index>>2);
     outl(apm_xbase,d);
 }
 
 static unsigned char inXR(int index)
 {
-    __svgalib_outbSR(0x1d,index>>2);
+    __svgalib_outseq(0x1d,index>>2);
     return inb(apm_xbase);
 }
 
@@ -77,14 +78,14 @@ static unsigned char inXR(int index)
 /* currently not used */
 static unsigned short inwXR(int index)
 {
-    __svgalib_outbSR(0x1d,index>>2);
+    __svgalib_outseq(0x1d,index>>2);
     return inw(apm_xbase);
 }
 #endif
 
 static unsigned long inlXR(int index)
 {
-    __svgalib_outbSR(0x1d,index>>2);
+    __svgalib_outseq(0x1d,index>>2);
     return inl(apm_xbase);
 }
 
@@ -129,15 +130,15 @@ static int apm_saveregs(unsigned char regs[])
 { unsigned long k;
 
     apm_unlock();		/* May be locked again by other programs (e.g. X) */
-    regs[APMREG_SAVE(2)] = __svgalib_inCR(0x19);
-    regs[APMREG_SAVE(3)] = __svgalib_inCR(0x1A);
-    regs[APMREG_SAVE(4)] = __svgalib_inCR(0x1B);
-    regs[APMREG_SAVE(5)] = __svgalib_inCR(0x1C);
-    regs[APMREG_SAVE(6)] = __svgalib_inCR(0x1D);
-    regs[APMREG_SAVE(7)] = __svgalib_inCR(0x1E);
+    regs[APMREG_SAVE(2)] = __svgalib_incrtc(0x19);
+    regs[APMREG_SAVE(3)] = __svgalib_incrtc(0x1A);
+    regs[APMREG_SAVE(4)] = __svgalib_incrtc(0x1B);
+    regs[APMREG_SAVE(5)] = __svgalib_incrtc(0x1C);
+    regs[APMREG_SAVE(6)] = __svgalib_incrtc(0x1D);
+    regs[APMREG_SAVE(7)] = __svgalib_incrtc(0x1E);
 
-    regs[APMREG_SAVE(0)] = __svgalib_inSR(0x1B);
-    regs[APMREG_SAVE(1)] = __svgalib_inSR(0x1C);
+    regs[APMREG_SAVE(0)] = __svgalib_inseq(0x1B);
+    regs[APMREG_SAVE(1)] = __svgalib_inseq(0x1C);
 
    regs[APMREG_SAVE(8)] = inXR(0x80) ;
    regs[APMREG_SAVE(9)] = inXR(0xc0) ;
@@ -169,14 +170,14 @@ static void apm_setregs(const unsigned char regs[], int mode)
     apm_unlock();		/* May be locked again by other programs (eg. X) */
     apm_setpage(0);
 
-    __svgalib_outSR(0x1b,regs[APMREG_SAVE(0)]);
-    __svgalib_outSR(0x1c,regs[APMREG_SAVE(1)]);
-    __svgalib_outCR(0x19,regs[APMREG_SAVE(2)]);
-    __svgalib_outCR(0x1a,regs[APMREG_SAVE(3)]);
-    __svgalib_outCR(0x1b,regs[APMREG_SAVE(4)]);
-    __svgalib_outCR(0x1c,regs[APMREG_SAVE(5)]);
-    __svgalib_outCR(0x1d,regs[APMREG_SAVE(6)]);
-    __svgalib_outCR(0x1e,regs[APMREG_SAVE(7)]);
+    __svgalib_outseq(0x1b,regs[APMREG_SAVE(0)]);
+    __svgalib_outseq(0x1c,regs[APMREG_SAVE(1)]);
+    __svgalib_outcrtc(0x19,regs[APMREG_SAVE(2)]);
+    __svgalib_outcrtc(0x1a,regs[APMREG_SAVE(3)]);
+    __svgalib_outcrtc(0x1b,regs[APMREG_SAVE(4)]);
+    __svgalib_outcrtc(0x1c,regs[APMREG_SAVE(5)]);
+    __svgalib_outcrtc(0x1d,regs[APMREG_SAVE(6)]);
+    __svgalib_outcrtc(0x1e,regs[APMREG_SAVE(7)]);
 
    if(apm_chiptype==AT3D) {
      k=regs[APMREG_SAVE(10)]+(regs[APMREG_SAVE(11)] << 8)+(regs[APMREG_SAVE(12)] << 16)+(regs[APMREG_SAVE(13)] << 24);
@@ -199,6 +200,10 @@ static int apm_modeavailable(int mode)
     struct info *info;
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
+
+    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
+
+    if((modeinfo->bitsPerPixel==24)&&(apm_chiptype==AT24))return 0;
   
     if ((mode < G640x480x256 )
 	|| mode == G720x348x2)
@@ -210,11 +215,6 @@ static int apm_modeavailable(int mode)
 
     modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
 
-    if((modeinfo->bitsPerPixel==24)&&(apm_chiptype==AT24)) {
-	free(modeinfo);
-        return 0;
-    }
-
     modetiming = malloc(sizeof(ModeTiming));
     if (__svgalib_getmodetiming(modetiming, modeinfo, cardspecs)) {
 	free(modetiming);
@@ -227,7 +227,6 @@ static int apm_modeavailable(int mode)
     return SVGADRV;
 }
 
-
 static unsigned comp_lmn(unsigned clock) ;
 /* Set a mode */
 
@@ -271,8 +270,8 @@ static void apm_initializemode(unsigned char *moderegs,
    * Enable VESA Super VGA memory organisation.
    * Also enable Linear Addressing.
    */
-    moderegs[APMREG_SAVE(0)] = 0;
-    moderegs[APMREG_SAVE(1)] = 21;
+  moderegs[APMREG_SAVE(0)] = 0;
+  moderegs[APMREG_SAVE(1)] = 21;
   /* Set banking register to zero. */
   moderegs[APMREG_SAVE(9)] = 0;
 
@@ -319,43 +318,44 @@ static void apm_initializemode(unsigned char *moderegs,
     moderegs[APMREG_SAVE(16)]=(k >> 16)&0xff;
     moderegs[APMREG_SAVE(17)]=(k >> 24)&0xff;   
     moderegs[MIS] |= 0xc;
-moderegs[MIS]=0xef; 
+    moderegs[MIS]=0xef; 
   /* Set up the RAMDAC registers. */
 
-  if (modeinfo->bitsPerPixel > 8)
+  if (modeinfo->bitsPerPixel > 8) {
     /* Get rid of white border. */
-    {moderegs[VGA_AR11] = 0;} else 
-{ int p,q,r;
+      moderegs[VGA_AR11] = 0;
+  } else { 
+      int p,q,r;
 
-moderegs[VGA_AR11]=0xff;
-p=moderegs[1]+((moderegs[APMREG_SAVE(4)]&2)<<7); 
-moderegs[2]=p&0xff;
-moderegs[APMREG_SAVE(4)]&=0xfd;
-moderegs[APMREG_SAVE(4)]|=(p&0x100)>>7;
-q=moderegs[4]+((moderegs[APMREG_SAVE(4)]&8)<<5); 
-r=moderegs[0]+4;
-moderegs[3]&=0xe0;
-moderegs[3]|=(r&0x1f);
-moderegs[5]&=0x7f;
-moderegs[5]|=(r&0x20)<<2;
+      moderegs[VGA_AR11]=0xff;
+      p=moderegs[1]+((moderegs[APMREG_SAVE(4)]&2)<<7); 
+      moderegs[2]=p&0xff;
+      moderegs[APMREG_SAVE(4)]&=0xfd;
+      moderegs[APMREG_SAVE(4)]|=(p&0x100)>>7;
+      q=moderegs[4]+((moderegs[APMREG_SAVE(4)]&8)<<5); 
+      r=moderegs[0]+4;
+      moderegs[3]&=0xe0;
+      moderegs[3]|=(r&0x1f);
+      moderegs[5]&=0x7f;
+      moderegs[5]|=(r&0x20)<<2;
+  
+      p=moderegs[0x12]+((moderegs[0x7]&2)<<7)+((moderegs[7]&0x40)<<3)+((moderegs[APMREG_SAVE(3)]&2)<<9);
+      moderegs[0x15]=p&0xff;
+      moderegs[0x7]&=0xf7;
+      moderegs[0x7]|=(p&0x100)>>5;
+      moderegs[0x9]&=0xdf;
+      moderegs[0x9]|=(p&0x200)>>4;
+      moderegs[APMREG_SAVE(3)]&=0xfb;
+      moderegs[APMREG_SAVE(3)]|=(p&0x400)>>7;
+      moderegs[0x16]=moderegs[6]+1;
+  }
 
-p=moderegs[0x12]+((moderegs[0x7]&2)<<7)+((moderegs[7]&0x40)<<3)+((moderegs[APMREG_SAVE(3)]&2)<<9);
-moderegs[0x15]=p&0xff;
-moderegs[0x7]&=0xf7;
-moderegs[0x7]|=(p&0x100)>>5;
-moderegs[0x9]&=0xdf;
-moderegs[0x9]|=(p&0x200)>>4;
-moderegs[APMREG_SAVE(3)]&=0xfb;
-moderegs[APMREG_SAVE(3)]|=(p&0x400)>>7;
-moderegs[0x16]=moderegs[6]+1;
-}
     moderegs[APMREG_SAVE(10)] = 0xe8;  
     moderegs[APMREG_SAVE(11)] = 0x01;  
     moderegs[APMREG_SAVE(12)] = 0x1f;   
     moderegs[APMREG_SAVE(13)] = 0x07;  
 
-
-apm_is_linear=0;
+    apm_is_linear=0;
 
 return ;
 
@@ -402,7 +402,6 @@ static int apm_setmode(int mode, int prv_mode)
 
     __svgalib_InitializeAcceleratorInterface(modeinfo);
 
-
     free(modeinfo);
     return 0;
 }
@@ -412,14 +411,8 @@ static int apm_setmode(int mode, int prv_mode)
 
 static void apm_unlock(void)
 {
-    int vgaIOBase, temp;
-
-    vgaIOBase = (inb(0x3CC) & 0x01) ? 0x3D0 : 0x3B0;
-    outb(vgaIOBase + 4, 0x11);
-    temp = inb(vgaIOBase + 5);
-    outb(vgaIOBase + 5, temp & 0x7F);
-    
-    __svgalib_outSR(0x10, 0x12);
+    __svgalib_outcrtc(0x11,__svgalib_incrtc(0x11)&0x7f);
+    __svgalib_outseq(0x10, 0x12);
 }
 
 
@@ -435,6 +428,7 @@ static void apm_lock(void)
 
 static int apm_test(void)
 {
+#ifdef OLD_TEST
     int     i;
     int	    oldSEQ10;
 
@@ -447,11 +441,6 @@ static int apm_test(void)
                                         correct... */
     char    idstring[]  = "       ";
 
-    if ((getenv("IOPERM") == NULL) && (iopl(3) < 0)) {
-	printf("svgalib(apm): Cannot get I/O permissions.\n");
-	exit(-1);
-    }
-
     /*
      * Warning! A fully compliant VGA will most probably interprete this as
      * __svgalib_outbSR(0x00, 0x02), hence stop the sequencer, to play safe
@@ -464,10 +453,6 @@ static int apm_test(void)
     for (i = 0; i < 7; i++)
       idstring[i] = __svgalib_inSR(0x11+i);
 
-    /* reset iopl, apm_init will raise it again */
-    if (getenv("IOPERM") == NULL)
-	iopl(0);
-
     /*
      * Just in case, restore any old setting & select SEQ00:
      */
@@ -477,6 +462,19 @@ static int apm_test(void)
     if (!memcmp(id_at3d, idstring, 7) || !memcmp(id_at24,idstring,7)) 
       { apm_init(0,0,0) ; return 1; }
     return 0;
+#else /* use PCI test */
+   int found=0;
+   unsigned long buf[64];
+    
+   found=__svgalib_pci_find_vendor_vga(0x1142,buf,0);
+   if(found)return 0;
+   if ((((buf[0]>>16)&0xffff)!=0x6424)&&
+       (((buf[0]>>16)&0xffff)!=0x643d)
+       )return 0;
+   apm_init(0,0,0);
+   return 1;
+#endif
+
 }
 
 
@@ -570,56 +568,32 @@ DriverSpecs __svgalib_apm_driverspecs =
 
 static int apm_init(int force, int par1, int par2)
 {
-        char    idstring[]  = "       ";
-
-    /* Get I/O priviledge */
-    if ((getenv("IOPERM") == NULL) && (iopl(3) < 0)) {
-	printf("svgalib(apm): Cannot get I/O permissions.\n");
-	exit(-1);
-    }
+    char    idstring[]  = "       ";
+    int found=0;
+    unsigned long buf[64];
 
     apm_unlock();
-    if (force) {
-	apm_memory = par1;
-	apm_chiptype = par2;
-    } else {
-        int     i;
-        char    id_ap6420[] = "Pro6420";
-        char    id_ap6422[] = "Pro6422";
-        char    id_at24[]   = "Pro6424";
-        char    id_at3d[]   = "ProAT3D"; /* Yeah, the manual could have been 
-                                        correct... */
 
-        __svgalib_outSR(0x10, 0x12);
-
-        for (i = 0; i < 7; i++)
-               idstring[i] = __svgalib_inSR(0x11+i);
-        if (!memcmp(id_ap6420, idstring, 7))
-           {
-         	apm_chiptype=AT6420;
-                return 1;
-           }
-        else if (!memcmp(id_ap6422, idstring, 7))
-           {
-         	apm_chiptype=AT6422;
-                return 1;
-           }
-        else if (!memcmp(id_at24, idstring, 7))
-           {
-         	apm_chiptype=AT24;
-           }
-        else if (!memcmp(id_at3d, idstring, 7))
-           {
-           	apm_chiptype=AT3D;            
-           }
-        /*else return 1;*/
-        apm_memory=__svgalib_inSR(0x20)*64-34; /* maybe will support accel some day */
+    found=__svgalib_pci_find_vendor_vga(0x1142,buf,0);
+    if(found)return 1;
+    switch((buf[0]>>16)&0xffff) {
+        case 0x6424: strcpy(idstring,"AT24"); break; 
+        case 0x643d: strcpy(idstring,"AT25/3D"); break; 
+        default: return 1; break;
     };
+
+    __svgalib_linear_mem_base=buf[4]&0xffffff00;
+    VGAMap=mmap(0,0x1000,PROT_READ|PROT_WRITE,MAP_SHARED,__svgalib_mem_fd,
+                __svgalib_linear_mem_base+0xfff000);
+/* not working
+    mm_io_mapio();
+*/
+    apm_memory=__svgalib_inseq(0x20)*64-34; /* maybe will support accel some day */
     if (__svgalib_driver_report) {
 	printf("Using Alliance driver, %.7s, %iKB.\n",idstring, apm_memory);
     }
 
-    apm_xbase= (__svgalib_inSR(0x1f) << 8 ) + __svgalib_inSR(0x1e);
+    apm_xbase= (__svgalib_inseq(0x1f) << 8 ) + __svgalib_inseq(0x1e);
     
     cardspecs = malloc(sizeof(CardSpecs));
     cardspecs->videoMemory = apm_memory;
@@ -630,7 +604,7 @@ static int apm_init(int force, int par1, int par2)
     cardspecs->maxPixelClock24bpp = 75000;
     cardspecs->maxPixelClock32bpp = 94500;
     cardspecs->flags = CLOCK_PROGRAMMABLE | INTERLACE_DIVIDE_VERT;
-    cardspecs->maxHorizontalCrtc = 4088;
+    cardspecs->maxHorizontalCrtc = 2040;
     cardspecs->maxPixelClock4bpp = 0;
     cardspecs->nClocks =0;
     cardspecs->clocks = NULL;
@@ -640,8 +614,6 @@ static int apm_init(int force, int par1, int par2)
     __svgalib_driverspecs = &__svgalib_apm_driverspecs;
     __svgalib_banked_mem_base=0xa0000;
     __svgalib_banked_mem_size=0x10000;
-    __svgalib_outSR(0x1d,0x193>>2); 
-    __svgalib_linear_mem_base=inb(apm_xbase+3)<<24 ;
     __svgalib_linear_mem_size=apm_memory*0x400;
     return 0;
 }
@@ -661,7 +633,7 @@ comp_lmn(unsigned clock)
 
     if(apm_chiptype==AT3D)fmax = 400000.0; else fmax=265000.0;
 /* The XFree86 driver says the max for AT24 is 250000,
-   but there is a much use clock (65MHZ), that is use 260000 fvco,
+   but there is a much used clock (65MHZ), that uses 260000 fvco,
    and the VESA bios on the card uses this clock, upped the max */   
   
   fref = 14318.0;
