@@ -124,6 +124,7 @@ static void banshee_setregs(const unsigned char regs[], int mode)
 
 }
 
+
 /* Return nonzero if mode is available */
 
 static int banshee_modeavailable(int mode)
@@ -132,6 +133,10 @@ static int banshee_modeavailable(int mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
+    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
+
+    if(modeinfo->greenWeight==5) return 0;
+
     if ((mode < G640x480x256 )
 	|| mode == G720x348x2)
 	return __svgalib_vga_driverspecs.modeavailable(mode);
@@ -142,14 +147,10 @@ static int banshee_modeavailable(int mode)
 
     modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
 
-    if((modeinfo->bitsPerPixel==16)&&(modeinfo->greenWeight==5)) {
-	free(modeinfo);
-        return 0;
-    }
-
     modetiming = malloc(sizeof(ModeTiming));
     if (__svgalib_getmodetiming(modetiming, modeinfo, cardspecs)) {
 	free(modetiming);
+	free(modeinfo);
 	return 0;
     }
     free(modetiming);
@@ -158,7 +159,7 @@ static int banshee_modeavailable(int mode)
     return SVGADRV;
 }
 
-static unsigned comp_lmn(int freq) ;
+static unsigned comp_lmn(unsigned clock) ;
 /* Set a mode */
 
 /* Local, called by banshee_setmode(). */
@@ -280,6 +281,51 @@ static int banshee_setmode(int mode, int prv_mode)
 
     banshee_initializemode(moderegs, &modetiming, modeinfo, mode);
 
+{
+#if 0
+static unsigned char regs[94] = {
+  0xA1,0x7F,0x7F,0x85,0x82,0x93,0x24,0xF5,	/* CR00-CR18 */
+  0x00,0x60,0x00,0x00,0x00,0x00,0x3F,0xFF,
+  0x02,0x00,0xFF,0x00,0x00,0xFF,0x25,0xC3,
+  0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,	/* AR00-AR15 */
+  0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F,
+  0x41,0xFF,0x0F,0x00,0x00,
+  0x00,0x00,0x00,0x00,0x00,0x40,0x05,0x0F,	/* GR00-GR05 */
+  0xFF,
+  0x03,0x01,0x0F,0x00,0x0E,			/* SR00-SR05 */
+  0xEF,						/* MISC_OUT  */
+  /* Extended (count = 0x22) */
+  0x00,0x00,0x4C,0x6C,0x00,0x00,0x05,0x28,
+  0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0x00,
+  0x00,0x00,0x81,0x0C,0x04,0x00,0x00,0x04,
+  0x30,0x00,0x40,0x11,0x00,0x00,0x00,0x00,
+  0x00,0x00
+};
+#endif
+#if 0
+unsigned char regs[94] = {
+  0xA3,0x7F,0x80,0x87,0x83,0x94,0x24,0xF5,	/* CR00-CR18 */
+  0x00,0x60,0x00,0x00,0x00,0x00,0x00,0x00,
+  0x03,0x00,0xFF,0x80,0x00,0xFF,0x25,0xE3,
+  0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,	/* AR00-AR15 */
+  0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F,
+  0x41,0x00,0x0F,0x00,0x00,
+  0x00,0x00,0x00,0x00,0x00,0x40,0x05,0x0F,	/* GR00-GR05 */
+  0xFF,
+  0x03,0x01,0x0F,0x00,0x0E,			/* SR00-SR05 */
+  0x2F,						/* MISC_OUT  */
+  /* Extended (count = 0x22) */
+  0x00,0x00,0x5D,0xE1,0x00,0x00,0x05,0x28,
+  0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0x00,
+  0x00,0x00,0x81,0x0C,0x04,0x00,0x00,0x04,
+  0x30,0x00,0x40,0x11,0x00,0x00,0x00,0x00,
+  0x10,0x00,
+};
+
+memcpy(moderegs,regs,94);
+#endif
+};
+
     __svgalib_setregs(moderegs);	/* Set standard regs. */
     banshee_setregs(moderegs, mode);		/* Set extended regs. */
     free(moderegs);
@@ -324,23 +370,13 @@ static void banshee_lock(void)
 
 static int banshee_test(void)
 {
-   int found,_ioperm=0;
+   int found;
    unsigned long buf[64];
 
-   if (getenv("IOPERM") == NULL) {
-      _ioperm=1;
-      if (iopl(3) < 0) {
-        printf("svgalib: banshee: cannot get I/O permissions\n");
-        exit(1);
-      }
-   }
    found=(!__svgalib_pci_find_vendor_vga(0x121a,buf,0))&&
           (((buf[0]>>16)==0x0003)||
-          ((buf[0]>>16)==0x0009)||
           ((buf[0]>>16)==0x0005));
     
-   if (_ioperm) iopl(0);
-
    if(found)banshee_init(0,0,0); 
    return found;
 }
@@ -351,9 +387,10 @@ static int banshee_test(void)
 
 static void banshee_setdisplaystart(int address)
 { 
-  outw(CRT_IC, ((address>>2) & 0x00FF00) | 0x0C);
-  outw(CRT_IC, (((address>>2) & 0x00FF) << 8) | 0x0D);
-  outl(banshee_io_base+0xe4,address);
+  address=address >> 2;
+  outw(CRT_IC, (address & 0x00FF00) | 0x0C);
+  outw(CRT_IC, ((address & 0x00FF) << 8) | 0x0D);
+  outl(banshee_io_base+0xe4,address>>2);
 
 }
 
@@ -427,7 +464,6 @@ static int banshee_init(int force, int par1, int par2)
 {
     unsigned long buf[64];
     int found=0;
-    int _ioperm=0;
 
     if (force) {
 	banshee_memory = par1;
@@ -436,20 +472,10 @@ static int banshee_init(int force, int par1, int par2)
 
     };
     
-    if (getenv("IOPERM") == NULL) {
-      _ioperm=1;
-      if (iopl(3) < 0) {
-        printf("svgalib: banshee: cannot get I/O permissions\n");
-        exit(1);
-      }
-    }
-     
     found=(!__svgalib_pci_find_vendor_vga(0x121a,buf,0))&&
             (((buf[0]>>16)==0x0003)||
-            ((buf[0]>>16)==0x0009)||
             ((buf[0]>>16)==0x0005));
     
-/*     if (_ioperm) iopl(0);*/
     if (found){
        banshee_linear_base=buf[5]&0xffffff00;
        banshee_io_base=buf[6]&0xff00;
@@ -474,18 +500,18 @@ static int banshee_init(int force, int par1, int par2)
     banshee_unlock();
     
     if (__svgalib_driver_report) {
-	printf("Using Banshee / Voodoo3 driver, %iKB.\n",banshee_memory);
+	printf("Using Banshee / Voodoo3 driver, %iKB. ",banshee_memory);
     }
     
     cardspecs = malloc(sizeof(CardSpecs));
     cardspecs->videoMemory = banshee_memory;
-    cardspecs->maxPixelClock4bpp = 270000;	
-    cardspecs->maxPixelClock8bpp = 270000;	
-    cardspecs->maxPixelClock16bpp = 270000;	
-    cardspecs->maxPixelClock24bpp = 270000;
-    cardspecs->maxPixelClock32bpp = 270000;
+    cardspecs->maxPixelClock4bpp = 250000;	
+    cardspecs->maxPixelClock8bpp = 250000;	
+    cardspecs->maxPixelClock16bpp = 250000;	
+    cardspecs->maxPixelClock24bpp = 250000;
+    cardspecs->maxPixelClock32bpp = 250000;
     cardspecs->flags = INTERLACE_DIVIDE_VERT | CLOCK_PROGRAMMABLE;
-    cardspecs->maxHorizontalCrtc = 4088;
+    cardspecs->maxHorizontalCrtc = 2040;
     cardspecs->maxPixelClock4bpp = 0;
     cardspecs->nClocks =0;
     cardspecs->mapClock = banshee_map_clock;
@@ -499,39 +525,45 @@ static int banshee_init(int force, int par1, int par2)
     return 0;
 }
 
-#define REFFREQ 14318.18
+#define WITHIN(v,c1,c2) (((v) >= (c1)) && ((v) <= (c2)))
 
 static unsigned
-comp_lmn(int freq)
+comp_lmn(unsigned clock)
 {
-  int m, n, k, best_m, best_n, best_k, f_cur, best_error;
+  int     n, m, l;
+  double  fvco;
+  double  fout;
+  double  fmax;
+  double  fref;
+  double  fvco_goal;
 
-  best_error=freq;
-  best_n=best_m=best_k=0;
-  for (n=1; n<256; n++) {
-    f_cur=REFFREQ*(n+2);
-    if (f_cur<freq) {
-      f_cur=f_cur/3;
-      if (freq-f_cur<best_error) {
-        best_error=freq-f_cur;
-        best_n=n;
-        best_m=1;
-        best_k=0;
-        continue;
-      }
-    }
-    for (m=1; m<64; m++) {
-      for (k=0; k<4; k++) {
-        f_cur=REFFREQ*(n+2)/(m+2)/(1<<k);
-        if (abs(f_cur-freq)<best_error) {
-          best_error=abs(f_cur-freq);
-          best_n=n;
-          best_m=m;
-          best_k=k;
-	}
+    fmax = 162000.0;
+
+  fref = 14318.18; 
+
+  for (m = 1; m <= 61; m++)
+  {
+    for (l = 3; l >= 0; l--)
+    {
+      for (n = 1; n <= 253; n++)
+      {
+        fout = ((double)(n + 2) * fref)/((double)(m + 2) * (1 << l));
+        fvco_goal = (double)clock * (double)(1 << l);
+        fvco = fout * (double)(1 << l);
+        if (!WITHIN(fvco, 0.995*fvco_goal, 1.005*fvco_goal))
+          continue;
+#if 0
+        if (!WITHIN(fvco, 32000.0, fmax))
+          continue;
+#endif
+#if 0
+printf("clock=%i n=%i m=%i l=%i",clock,n,m,l);
+#endif
+        return (n << 8) | (m<<2) | l;
       }
     }
   }
-  return (best_n << 8) | (best_m<<2) | best_k;
+printf("BANSHEE driver: Can't do clock=%i\n",clock);
+  return 0;
 }
 
