@@ -88,11 +88,6 @@ extern DriverSpecs __svgalib_rage_driverspecs;
 extern DriverSpecs __svgalib_banshee_driverspecs;
 extern DriverSpecs __svgalib_sis_driverspecs;
 extern DriverSpecs __svgalib_i740_driverspecs;
-extern DriverSpecs __svgalib_laguna_driverspecs;
-extern DriverSpecs __svgalib_fbdev_driverspecs;
-extern DriverSpecs __svgalib_g400_driverspecs;
-extern DriverSpecs __svgalib_r128_driverspecs;
-extern DriverSpecs __svgalib_savage_driverspecs;
 
 extern DriverSpecs *__svgalib_driverspecs;
 extern DriverSpecs *__svgalib_driverspecslist[];
@@ -144,7 +139,7 @@ void __svgalib_read_options(char **commands, char *(*func) (int ind, int mode));
    ** the last entry is marked by 
    **  
    **    <any number>  <NULL>
- */
+*/
 
 typedef struct {
     unsigned short mode_number;
@@ -165,7 +160,6 @@ extern int __svgalib_hicolor(int dac_type, int mode);
 /* Enters hicolor mode - 0 for no hi, 1 for 15 bit, 2 for 16, 3 for 24 */
 /* For any modes it doesn't know about, etc, it attempts to turn hicolor off. */
 
-
 #define STD_DAC		0
 #define HI15_DAC	1
 #define HI16_DAC	2
