@@ -9,6 +9,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <math.h>
+#include <sys/mman.h>
 #include "vga.h"
 #include "libvga.h"
 #include "driver.h"
@@ -18,6 +19,7 @@
 #include "accel.h"
 #include "vgapci.h"
 #include "i740_reg.h"
+#include "vgammvgaio.h"
 
 typedef struct {
     unsigned char DisplayControl;
@@ -57,13 +59,38 @@ void __svgalib_i740accel_init(AccelSpecs * accelspecs, int bpp, int width_in_pix
 static int i740_memory, I740HasSGRAM;
 static int i740_is_linear, i740_linear_base, i740_mmio_base;
 
-
 static CardSpecs *cardspecs;
 
+static void i740_outxr_io(int index, int val)
+{
+    outb(XRX,index);
+    outb(XRX+1,val);
+}
+
+static int i740_inxr_io(int index)
+{
+    outb(XRX,index);
+    return inb(XRX+1);
+}
+
+static void i740_outxr_mm(int index, int val)
+{
+    *(VGAMap+XRX)=index;
+    *(VGAMap+XRX+1)=val;
+}
+
+static int i740_inxr_mm(int index)
+{
+    *(VGAMap+XRX)=index;
+    return *(VGAMap+XRX+1);
+}
+
+static void (*i740_outxr)(int,int)=i740_outxr_io;
+static int (*i740_inxr)(int)=i740_inxr_io;
+
 static void i740_setpage(int page)
 {
-   outb(XRX,14);
-   outb(XRX+1,page);
+   i740_outxr(14,page);
 }
 
 static int __svgalib_i740_inlinearmode(void)
@@ -79,7 +106,7 @@ static void i740_getmodeinfo(int mode, vga_modeinfo *modeinfo)
     if(modeinfo->colors==16)return;
 
     modeinfo->maxpixels = i740_memory*1024/modeinfo->bytesperpixel;
-    modeinfo->maxlogicalwidth = 4088;
+    modeinfo->maxlogicalwidth = 8184;
     modeinfo->startaddressrange = i740_memory * 1024 - 1;
     modeinfo->haveblit = 0;
     modeinfo->flags &= ~HAVE_RWPAGE;
@@ -105,30 +132,30 @@ static int i740_saveregs(unsigned char regs[])
      * into the fields of the vgaI740Rec structure goes here.
      */
 
-    outb(XRX, IO_CTNL);           save->IOControl = inb(XRX+1);
-    outb(XRX, ADDRESS_MAPPING);   save->AddressMapping = inb(XRX+1);
-    outb(XRX, BITBLT_CNTL);       save->BitBLTControl = inb(XRX+1);
-    outb(XRX, VCLK2_VCO_M);       save->VideoClk2_M = inb(XRX+1);
-    outb(XRX, VCLK2_VCO_N);       save->VideoClk2_N = inb(XRX+1);
-    outb(XRX, VCLK2_VCO_MN_MSBS); save->VideoClk2_MN_MSBs = inb(XRX+1);
-    outb(XRX, VCLK2_VCO_DIV_SEL); save->VideoClk2_DivisorSel = inb(XRX+1);
-    outb(XRX, PLL_CNTL);          save->PLLControl = inb(XRX+1);
+    i740_outxr(IO_CTNL, save->IOControl);
+    i740_outxr(ADDRESS_MAPPING, save->AddressMapping);
+    i740_outxr(BITBLT_CNTL, save->BitBLTControl);
+    i740_outxr(VCLK2_VCO_M, save->VideoClk2_M);
+    i740_outxr(VCLK2_VCO_N, save->VideoClk2_N);
+    i740_outxr(VCLK2_VCO_MN_MSBS, save->VideoClk2_MN_MSBs);
+    i740_outxr(VCLK2_VCO_DIV_SEL, save->VideoClk2_DivisorSel);
+    i740_outxr(PLL_CNTL, save->PLLControl);
 
-    save->ExtVertTotal = __svgalib_inCR(EXT_VERT_TOTAL);
-    save->ExtVertDispEnd = __svgalib_inCR(EXT_VERT_DISPLAY);
-    save->ExtVertSyncStart = __svgalib_inCR(EXT_VERT_SYNC_START);
-    save->ExtVertBlankStart = __svgalib_inCR(EXT_VERT_BLANK_START);
-    save->ExtHorizTotal = __svgalib_inCR(EXT_HORIZ_TOTAL);
-    save->ExtHorizBlank = __svgalib_inCR(EXT_HORIZ_BLANK);
-    save->ExtOffset = __svgalib_inCR(EXT_OFFSET);
-    save->InterlaceControl = __svgalib_inCR(INTERLACE_CNTL);
-    save->ExtStartAddr = __svgalib_inCR(EXT_START_ADDR);
-    save->ExtStartAddrHi = __svgalib_inCR(EXT_START_ADDR_HI);
+    save->ExtVertTotal = __svgalib_incrtc(EXT_VERT_TOTAL);
+    save->ExtVertDispEnd = __svgalib_incrtc(EXT_VERT_DISPLAY);
+    save->ExtVertSyncStart = __svgalib_incrtc(EXT_VERT_SYNC_START);
+    save->ExtVertBlankStart = __svgalib_incrtc(EXT_VERT_BLANK_START);
+    save->ExtHorizTotal = __svgalib_incrtc(EXT_HORIZ_TOTAL);
+    save->ExtHorizBlank = __svgalib_incrtc(EXT_HORIZ_BLANK);
+    save->ExtOffset = __svgalib_incrtc(EXT_OFFSET);
+    save->InterlaceControl = __svgalib_incrtc(INTERLACE_CNTL);
+    save->ExtStartAddr = __svgalib_incrtc(EXT_START_ADDR);
+    save->ExtStartAddrHi = __svgalib_incrtc(EXT_START_ADDR_HI);
 
-    outb(XRX, PIXPIPE_CONFIG_0); save->PixelPipeCfg0 = inb(XRX+1);
-    outb(XRX, PIXPIPE_CONFIG_1); save->PixelPipeCfg1 = inb(XRX+1);
-    outb(XRX, PIXPIPE_CONFIG_2); save->PixelPipeCfg2 = inb(XRX+1);
-    outb(XRX, DISPLAY_CNTL);     save->DisplayControl = inb(XRX+1);
+    i740_outxr(PIXPIPE_CONFIG_0, save->PixelPipeCfg0);
+    i740_outxr(PIXPIPE_CONFIG_1, save->PixelPipeCfg1);
+    i740_outxr(PIXPIPE_CONFIG_2, save->PixelPipeCfg2);
+    i740_outxr(DISPLAY_CNTL, save->DisplayControl);
 
     save->LMI_FIFO_Watermark = INREG(FWATER_BLC);
 
@@ -145,87 +172,86 @@ static void i740_setregs(const unsigned char regs[], int mode)
     i740_unlock();		
     restore = (vgaI740Ptr)(regs+VGA_TOTAL_REGS);
 
-
-    outb(XRX, DRAM_EXT_CNTL); outb(XRX+1, DRAM_REFRESH_DISABLE);
+    i740_outxr(DRAM_EXT_CNTL, DRAM_REFRESH_DISABLE);
 
     usleep(1000); /* Wait 1 ms */
 
     /* Write the M, N and P values */
-    outb(XRX, VCLK2_VCO_M);       outb(XRX+1, restore->VideoClk2_M);
-    outb(XRX, VCLK2_VCO_N);       outb(XRX+1, restore->VideoClk2_N);
-    outb(XRX, VCLK2_VCO_MN_MSBS); outb(XRX+1, restore->VideoClk2_MN_MSBs);
-    outb(XRX, VCLK2_VCO_DIV_SEL); outb(XRX+1, restore->VideoClk2_DivisorSel);
+    i740_outxr(VCLK2_VCO_M, restore->VideoClk2_M);
+    i740_outxr(VCLK2_VCO_N, restore->VideoClk2_N);
+    i740_outxr(VCLK2_VCO_MN_MSBS, restore->VideoClk2_MN_MSBs);
+    i740_outxr(VCLK2_VCO_DIV_SEL, restore->VideoClk2_DivisorSel);
 
-    outb(XRX, PIXPIPE_CONFIG_0); temp = inb(XRX+1);
+    temp = i740_inxr(PIXPIPE_CONFIG_0);
     temp &= 0x7F; /* Save all but the 8 bit dac mode bit */
     temp |= (restore->PixelPipeCfg0 & DAC_8_BIT);
-    outb(XRX, PIXPIPE_CONFIG_0); outb(XRX+1, temp);
+    i740_outxr(PIXPIPE_CONFIG_0, temp);
 
-    __svgalib_outCR(EXT_VERT_TOTAL, restore->ExtVertTotal);
+    __svgalib_outcrtc(EXT_VERT_TOTAL, restore->ExtVertTotal);
 
-    __svgalib_outCR(EXT_VERT_DISPLAY, restore->ExtVertDispEnd);
+    __svgalib_outcrtc(EXT_VERT_DISPLAY, restore->ExtVertDispEnd);
 
-    __svgalib_outCR(EXT_VERT_SYNC_START, restore->ExtVertSyncStart);
+    __svgalib_outcrtc(EXT_VERT_SYNC_START, restore->ExtVertSyncStart);
 
-    __svgalib_outCR(EXT_VERT_BLANK_START, restore->ExtVertBlankStart);
+    __svgalib_outcrtc(EXT_VERT_BLANK_START, restore->ExtVertBlankStart);
 
-    __svgalib_outCR(EXT_HORIZ_TOTAL, restore->ExtHorizTotal);
+    __svgalib_outcrtc(EXT_HORIZ_TOTAL, restore->ExtHorizTotal);
 
-    __svgalib_outCR(EXT_HORIZ_BLANK, restore->ExtHorizBlank);
+    __svgalib_outcrtc(EXT_HORIZ_BLANK, restore->ExtHorizBlank);
 
-    __svgalib_outCR(EXT_OFFSET, restore->ExtOffset);
+    __svgalib_outcrtc(EXT_OFFSET, restore->ExtOffset);
 
-    __svgalib_outCR(EXT_START_ADDR, restore->ExtStartAddr);
-    __svgalib_outCR(EXT_START_ADDR_HI, restore->ExtStartAddrHi);
+    __svgalib_outcrtc(EXT_START_ADDR, restore->ExtStartAddr);
+    __svgalib_outcrtc(EXT_START_ADDR_HI, restore->ExtStartAddrHi);
 
     temp=__svgalib_inCR(INTERLACE_CNTL); 
     temp &= ~INTERLACE_ENABLE;
     temp |= restore->InterlaceControl;
-    __svgalib_outCR(INTERLACE_CNTL, temp);
+    __svgalib_outcrtc(INTERLACE_CNTL, temp);
 
-    outb(XRX, ADDRESS_MAPPING); temp = inb(XRX+1);
+    temp = i740_inxr(ADDRESS_MAPPING);
     temp &= 0xE0; /* Save reserved bits 7:5 */
     temp |= restore->AddressMapping;
-    outb(XRX, ADDRESS_MAPPING); outb(XRX+1, temp);
+    i740_outxr(ADDRESS_MAPPING, temp);
 
-    outb(XRX, IO_CTNL); temp = inb(XRX+1);
+    temp = i740_inxr(IO_CTNL);
     temp &= ~(EXTENDED_ATTR_CNTL|EXTENDED_CRTC_CNTL); 
     temp |= restore->IOControl;
-    outb(XRX, IO_CTNL); outb(XRX+1, temp);
+    i740_outxr(IO_CTNL, temp);
 
-    outb(XRX, BITBLT_CNTL); temp = inb(XRX+1);
+    temp = i740_inxr(BITBLT_CNTL);
     temp &= ~COLEXP_MODE;
     temp |= restore->BitBLTControl;
-    outb(XRX, BITBLT_CNTL); outb(XRX+1, temp);
+    i740_outxr(BITBLT_CNTL, temp);
 
-    outb(XRX, DISPLAY_CNTL); temp = inb(XRX+1);
+    temp = i740_inxr(DISPLAY_CNTL);
     temp &= ~(VGA_WRAP_MODE | GUI_MODE);
     temp |= restore->DisplayControl;
-    outb(XRX, DISPLAY_CNTL); outb(XRX+1, temp);
+    i740_outxr(DISPLAY_CNTL, temp);
 
-    outb(XRX, PIXPIPE_CONFIG_0); temp = inb(XRX+1);
+    temp = i740_inxr(PIXPIPE_CONFIG_0);
     temp &= 0x64; /* Save reserved bits 6:5,2 */
     temp |= restore->PixelPipeCfg0;
-    outb(XRX, PIXPIPE_CONFIG_0); outb(XRX+1, temp);
+    i740_outxr(PIXPIPE_CONFIG_0, temp);
 
-    outb(XRX, PIXPIPE_CONFIG_2); temp = inb(XRX+1);
+    temp = i740_inxr(PIXPIPE_CONFIG_2);
     temp &= 0xF3; /* Save reserved bits 7:4,1:0 */
     temp |= restore->PixelPipeCfg2;
-    outb(XRX, PIXPIPE_CONFIG_2); outb(XRX+1, temp);
+    i740_outxr(PIXPIPE_CONFIG_2, temp);
 
-    outb(XRX, PLL_CNTL); temp = inb(XRX+1);
+    temp = i740_inxr(PLL_CNTL);
     temp &= ~PLL_MEMCLK_SEL;
 #if 1
     temp = restore->PLLControl; /* To fix the 2.3X BIOS problem */
 #else
     temp |= restore->PLLControl;
 #endif
-    outb(XRX, PLL_CNTL); outb(XRX+1, temp);
+    i740_outxr(PLL_CNTL, temp);
 
-    outb(XRX, PIXPIPE_CONFIG_1); temp = inb(XRX+1);
+    temp = i740_inxr(PIXPIPE_CONFIG_1);
     temp &= ~DISPLAY_COLOR_MODE;
     temp |= restore->PixelPipeCfg1;
-    outb(XRX, PIXPIPE_CONFIG_1); outb(XRX+1, temp);
+    i740_outxr(PIXPIPE_CONFIG_1, temp);
 
     temp = INREG(FWATER_BLC);
     temp &= ~(LMI_BURST_LENGTH | LMI_FIFO_WATERMARK);
@@ -233,7 +259,7 @@ static void i740_setregs(const unsigned char regs[], int mode)
     OUTREG(FWATER_BLC, temp);
 
     /* Turn on DRAM Refresh */
-    outb(XRX, DRAM_EXT_CNTL); outb(XRX+1, DRAM_REFRESH_60HZ);
+    i740_outxr(DRAM_EXT_CNTL, DRAM_REFRESH_60HZ);
 
     usleep(50000);
 
@@ -247,6 +273,8 @@ static int i740_modeavailable(int mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
+    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
+
     if ((mode < G640x480x256 )
 	|| mode == G720x348x2)
 	return __svgalib_vga_driverspecs.modeavailable(mode);
@@ -431,7 +459,7 @@ static void i740_initializemode(unsigned char *moderegs,
     new = (vgaI740Ptr)(moderegs+VGA_TOTAL_REGS);
 
     moderegs[0x13] = modeinfo->lineWidth >> 3;
-    new->ExtOffset      = modeinfo->lineWidth >> 11;
+    new->ExtOffset = modeinfo->lineWidth >> 11;
 
     switch (modeinfo->bitsPerPixel) {
         case 8:
@@ -520,7 +548,6 @@ static int i740_setmode(int mode, int prv_mode)
     ModeInfo *modeinfo;
 
     if ((mode < G640x480x256)||(mode==G720x348x2)) {
-
 	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
     }
     if (!i740_modeavailable(mode))
@@ -555,23 +582,12 @@ static int i740_setmode(int mode, int prv_mode)
 
 static void i740_unlock(void)
 {
-    int vgaIOBase, temp;
-
-    vgaIOBase = (inb(0x3CC) & 0x01) ? 0x3D0 : 0x3B0;
-    outb(vgaIOBase + 4, 0x11);
-    temp = inb(vgaIOBase + 5);
-    outb(vgaIOBase + 5, temp & 0x7F);
+    __svgalib_outcrtc(0x11,__svgalib_incrtc(0x11)&0x7f);
     
 }
 
 static void i740_lock(void)
 {
-    int vgaIOBase, temp;
-
-    vgaIOBase = (inb(0x3CC) & 0x01) ? 0x3D0 : 0x3B0;
-    outb(vgaIOBase + 4, 0x11);
-    temp = inb(vgaIOBase + 5);
-    outb(vgaIOBase + 5, temp & 0x7F);   
 }
 
 
@@ -583,21 +599,11 @@ static void i740_lock(void)
 
 static int i740_test(void)
 {
-    int _ioperm=0,found;
+    int found;
     long buf[64];
     
-    if (getenv("IOPERM") == NULL) {
-      _ioperm=1;
-      if (iopl(3) < 0) {
-        printf("svgalib: cannot get I/O permissions\n");
-        exit(1);
-      }
-    }
-    
     found=__svgalib_pci_find_vendor_vga(VENDOR_ID,buf,0);
     
-    if (_ioperm) iopl(0);
-    
     if(!found&&
        ((((buf[0]>>16)&0xffff)==I740_ID_PCI)||
         (((buf[0]>>16)&0xffff)==I740_ID_AGP))){
@@ -615,10 +621,10 @@ static int i740_test(void)
 static void i740_setdisplaystart(int address)
 { 
   address=address >> 2;
-  outw(CRT_IC, (address & 0x00FF00) | 0x0C);
-  outw(CRT_IC, ((address & 0x00FF) << 8) | 0x0D);
-  __svgalib_outCR(EXT_START_ADDR_HI,(address&0x3fc00000)>>22);
-  __svgalib_outCR(EXT_START_ADDR,((address&0x3f0000)>>16)|EXT_START_ADDR_ENABLE);
+  __svgalib_outcrtc(0x0c, address & 0xFF00);
+  __svgalib_outcrtc(0x0d, address & 0x00FF);
+  __svgalib_outcrtc(EXT_START_ADDR_HI,(address&0x3fc00000)>>22);
+  __svgalib_outcrtc(EXT_START_ADDR,((address&0x3f0000)>>16)|EXT_START_ADDR_ENABLE);
 }
 
 
@@ -629,7 +635,8 @@ static void i740_setlogicalwidth(int width)
 {   
     int offset = width >> 3;
  
-    __svgalib_outCR(0x13,offset&0xff);
+    __svgalib_outcrtc(0x13,offset&0xff);
+    __svgalib_outCR(EXT_OFFSET, offset>>8);
 }
 
 static int i740_linear(int op, int param)
@@ -692,40 +699,37 @@ static int i740_init(int force, int par1, int par2)
 {
     unsigned long buf[64];
     int found=0;
-    int _ioperm=0;
     int temp;
     
+    i740_inxr=i740_inxr_io;
+    i740_outxr=i740_outxr_io;
+
     i740_unlock();
-    if (force) {
-	i740_memory = par1;
-/*        i740_chiptype = par2;*/
-    } else {
-	outb(XRX, DRAM_ROW_TYPE);
-	if ((inb(XRX+1) & DRAM_ROW_1) == DRAM_ROW_1_SDRAM)
-	    outb(XRX, DRAM_ROW_BNDRY_1);
-	else
-	    outb(XRX, DRAM_ROW_BNDRY_0);
-        i740_memory = inb(XRX+1)*1024;
-    };
 
-    outb(XRX, DRAM_ROW_CNTL_LO); temp = inb(XRX+1);
-    I740HasSGRAM = !((temp & DRAM_RAS_TIMING) || (temp & DRAM_RAS_PRECHARGE));
-
-    if (getenv("IOPERM") == NULL) {
-      _ioperm=1;
-      if (iopl(3) < 0) {
-        printf("svgalib: cannot get I/O permissions\n");
-        exit(1);
-      }
-    }
     found=__svgalib_pci_find_vendor_vga(VENDOR_ID,buf,0);
-    if (_ioperm) iopl(0);
     i740_linear_base=0;
     if(!found&&((((buf[0]>>16)&0xffff)==I740_ID_PCI)||(((buf[0]>>16)&0xffff)==I740_ID_AGP))){
        i740_linear_base=buf[4]&0xffffff00;
        i740_mmio_base=buf[5]&0xffffff00;
     };
 
+    VGAMap=mmap(0,0x1000,PROT_READ|PROT_WRITE,MAP_SHARED,__svgalib_mem_fd,i740_mmio_base);
+    mm_io_mapio();
+    i740_inxr=i740_inxr_mm;
+    i740_outxr=i740_outxr_mm;
+    
+    if (force) {
+	i740_memory = par1;
+/*        i740_chiptype = par2;*/
+    } else {
+        if((i740_inxr(DRAM_ROW_TYPE)&DRAM_ROW_1) == DRAM_ROW_1_SDRAM)
+            i740_memory=i740_inxr(DRAM_ROW_BNDRY_1)*1024; else
+            i740_memory=i740_inxr(DRAM_ROW_BNDRY_0)*1024; 
+    };
+
+    temp=i740_inxr(DRAM_ROW_CNTL_LO);
+    I740HasSGRAM = !((temp & DRAM_RAS_TIMING) || (temp & DRAM_RAS_PRECHARGE));
+
     if (__svgalib_driver_report) {
 	printf("Using I740 driver, %iKB.\n",i740_memory);
     };
@@ -738,7 +742,7 @@ static int i740_init(int force, int par1, int par2)
     cardspecs->maxPixelClock24bpp = 128000;
     cardspecs->maxPixelClock32bpp = 86000;
     cardspecs->flags = CLOCK_PROGRAMMABLE;
-    cardspecs->maxHorizontalCrtc = 4088;
+    cardspecs->maxHorizontalCrtc = 2040;
     cardspecs->maxPixelClock4bpp = 0;
     cardspecs->nClocks =0;
     cardspecs->mapClock = i740_map_clock;
