@@ -11,7 +11,6 @@
  * July 3 1998: Added keyboard remapping support - brion
 */
 
-
 /*
  * Keyboard I/O based on showkey.c from kbd-0.84 package.
  * This is an initial version, it isn't very safe yet since it only catches
@@ -322,25 +321,30 @@ int keyboard_init_return_fd(void) {
 
     /* Install default keyboard handler. */
     __svgalib_keyboard_eventhandler = default_handler;
-
-    __svgalib_open_devconsole();
-    __svgalib_kbd_fd = __svgalib_tty_fd; /* We are initialized. */
-
-    if (ioctl(__svgalib_kbd_fd, KDGKBMODE, &oldkbmode)) {
-	printf("svgalib: cannot get keyboard mode.\n");
-	return -1;
+    
+    if(__svgalib_novccontrol) {
+        __svgalib_kbd_fd = open("/tmp/svga_kbd",O_RDONLY);
+        freopen("/tmp/svga_tty","r",stdin);
+    } else {
+        __svgalib_open_devconsole();
+        __svgalib_kbd_fd = __svgalib_tty_fd; /* We are initialized. */
+    
+        if (ioctl(__svgalib_kbd_fd, KDGKBMODE, &oldkbmode)) {
+	    printf("svgalib: cannot get keyboard mode.\n");
+	    return -1;
+        }
+        tcgetattr(__svgalib_kbd_fd, &oldkbdtermios);
+        newkbdtermios = oldkbdtermios;
+    
+        newkbdtermios.c_lflag &= ~(ICANON | ECHO | ISIG);
+        newkbdtermios.c_iflag &= ~(ISTRIP | IGNCR | ICRNL | INLCR | IXOFF | IXON);
+        newkbdtermios.c_cc[VMIN] = 0;	/* Making these 0 seems to have the */
+        newkbdtermios.c_cc[VTIME] = 0;	/* desired effect. */
+    
+        tcsetattr(__svgalib_kbd_fd, TCSAFLUSH, &newkbdtermios);
+    
+        ioctl(__svgalib_kbd_fd, KDSKBMODE, K_MEDIUMRAW);
     }
-    tcgetattr(__svgalib_kbd_fd, &oldkbdtermios);
-    newkbdtermios = oldkbdtermios;
-
-    newkbdtermios.c_lflag &= ~(ICANON | ECHO | ISIG);
-    newkbdtermios.c_iflag &= ~(ISTRIP | IGNCR | ICRNL | INLCR | IXOFF | IXON);
-    newkbdtermios.c_cc[VMIN] = 0;	/* Making these 0 seems to have the */
-    newkbdtermios.c_cc[VTIME] = 0;	/* desired effect. */
-
-    tcsetattr(__svgalib_kbd_fd, TCSAFLUSH, &newkbdtermios);
-
-    ioctl(__svgalib_kbd_fd, KDSKBMODE, K_MEDIUMRAW);
 
     keyboard_clearstate();
 
@@ -378,10 +382,11 @@ void keyboard_close(void) {
       free(fake_mouse_events);
       fake_mouse_events = NULL;
     }
-
-    ioctl(__svgalib_kbd_fd, KDSKBMODE, oldkbmode);
-    tcsetattr(__svgalib_kbd_fd, 0, &oldkbdtermios);
-
+    if(!__svgalib_novccontrol) {
+        ioctl(__svgalib_kbd_fd, KDSKBMODE, oldkbmode);
+        tcsetattr(__svgalib_kbd_fd, 0, &oldkbdtermios);
+    }
+    
     __svgalib_kbd_fd = -1;
 }
 
@@ -482,7 +487,7 @@ static int keyboard_getevents(int wait)
              	win_state = is_pressed;
 		break;
 
-	  case SCANCODE_F1 ... SCANCODE_F10:
+	  case SCANCODE_F1...SCANCODE_F10:
 		functionkey_state = (is_pressed)  ?
 		  functionkey_state |   1 << (scancode - SCANCODE_F1)   :
 		  functionkey_state & ~(1 << (scancode - SCANCODE_F1));
@@ -574,7 +579,8 @@ static int keyboard_getevents(int wait)
 
 	if (ctrl_state && c_state && !(translatemode & DONT_CATCH_CTRLC))
 	    raise(SIGINT);
-	if ((alt_state && functionkey_state)||(win_state && functionkey_state)) {
+	if (((alt_state && functionkey_state)||(win_state && functionkey_state)) &&
+             !__svgalib_novccontrol){
 	    /* VT switch. */
 	    /* *** what about F11 & F12? */
 	    int j, vt = 0;
