@@ -1,3 +1,4 @@
+
 /* SVGAlib, Copyright 1993 Harm Hanemaayer */
 /* VGAlib version 1.2 - (c) 1993 Tommy Frandsen */
 /* partially copyrighted (C) 1993 by Hartmut Schirmer */
@@ -74,9 +75,7 @@
 
 /* Shorthands for internal variables and functions */
 #define CI	__svgalib_cur_info
-#ifdef __alpha__
 #define SM	__svgalib_sparse_mem
-#endif
 #define GM	__svgalib_graph_mem
 #define CM	__svgalib_cur_mode
 #define VMEM	__svgalib_videomemoryused
@@ -162,15 +161,13 @@ extern unsigned char *SM;	/* sparse graphics memory frame */
 extern int MODEX;		/* TRUE after vga_setmodeX() */
 extern int MODEFLAGS;		/* copy of flags of current modeinfo->flags */
 
-#ifdef BACKGROUND
-extern int __svgalib_virtual_mem_fd;
-#endif
 extern int __svgalib_mem_fd;
 extern int __svgalib_tty_fd;
 extern int __svgalib_nosigint;
 extern int __svgalib_mouse_fd;
 extern int __svgalib_kbd_fd;
 extern int __svgalib_runinbackground;
+extern int __svgalib_vgacolormode;
 
 extern unsigned char __svgalib_novga;
 extern unsigned char __svgalib_textprog;
@@ -184,8 +181,20 @@ extern char *__joystick_devicenames[4];
 
 /* --------------------- Function definitions shared by library modules */
 
-extern int __svgalib_inmisc(void);
-extern void __svgalib_outmisc(int);
+extern int (*__svgalib_inmisc)(void);
+extern void (*__svgalib_outmisc)(int);
+extern int (*__svgalib_incrtc)(int);
+extern void (*__svgalib_outcrtc)(int,int);
+extern int (*__svgalib_inseq)(int);
+extern void (*__svgalib_outseq)(int,int);
+extern int  (*__svgalib_ingra)(int);
+extern void (*__svgalib_outgra)(int,int);
+extern int  (*__svgalib_inatt)(int);
+extern void (*__svgalib_outatt)(int,int);
+extern void (*__svgalib_attscreen)(int);
+extern void (*__svgalib_inpal)(int,int*,int*,int*);
+extern void (*__svgalib_outpal)(int,int,int,int);
+extern int (*__svgalib_inis1)(void);
 
 extern int __svgalib_setregs(const unsigned char *regs);
 extern int __svgalib_saveregs(unsigned char *regs);
@@ -209,6 +218,7 @@ extern int __svgalib_VESA_textmode;
 extern unsigned char __svgalib_vesatext;
 extern int __svgalib_mapkeyname(const char *keyname);
 extern void __svgalib_mouse_update_keymap(void);
+extern int __svgalib_vgacolor(void);
 
 #if 0 
 /* remove this part ? */
@@ -220,31 +230,6 @@ extern void __svgalib_acquirevt_signal(int n);
 
 #include <asm/io.h>
 
-static __inline__ void port_out(int value, int port)
-{
-    _outb(value, port);
-}
-
-static __inline__ void port_outw(int value, int port)
-{
-    _outw(value, port);
-}
-
-static __inline__ int port_in(int port)
-{
-    return _inb(port);
-}
-
-static __inline__ int port_inw(int port)
-{
-    return _inw(port);
-}
-
-#undef inb
-#undef inw
-#undef outb
-#undef outw
-
 extern unsigned long vga_readb(unsigned long base, unsigned long off);
 extern unsigned long vga_readw(unsigned long base, unsigned long off);
 extern void vga_writeb(unsigned char b, unsigned long base, unsigned long off);
@@ -270,51 +255,6 @@ extern void vga_writew(unsigned short b, unsigned long base, unsigned long off);
 
 #else
 
-static __inline__ void port_out(int value, int port)
-{
-    __asm__ volatile ("outb %0,%1"
-	      ::"a" ((unsigned char) value), "d"((unsigned short) port));
-}
-
-static __inline__ void port_outw(int value, int port)
-{
-    __asm__ volatile ("outw %0,%1"
-	     ::"a" ((unsigned short) value), "d"((unsigned short) port));
-}
-
-static __inline__ void port_outl(int value, int port)
-{
-    __asm__ volatile ("outl %0,%1"
-             ::"a" ((unsigned long)value), "d" ((unsigned short) port));
-}
-
-static __inline__ int port_in(int port)
-{
-    unsigned char value;
-    __asm__ volatile ("inb %1,%0"
-		      :"=a" (value)
-		      :"d"((unsigned short) port));
-    return value;
-}
-
-static __inline__ int port_inw(int port)
-{
-    unsigned short value;
-    __asm__ volatile ("inw %1,%0"
-		      :"=a" (value)
-		      :"d"((unsigned short) port));
-    return value;
-}
-
-static __inline__ int port_inl(int port)
-{
-    unsigned int value;
-    __asm__ volatile("inl %1,%0" :
-               	     "=a" (value) :
-                     "d" ((unsigned short)port));
-    return value;
-}
-
 #define gr_readb(off)		(((volatile unsigned char *)GM)[(off)])
 #define gr_readw(off)		(*(volatile unsigned short*)((GM)+(off)))
 #define gr_readl(off)		(*(volatile unsigned long*)((GM)+(off)))
@@ -322,8 +262,14 @@ static __inline__ int port_inl(int port)
 #define gr_writew(v,off)	(*(unsigned short*)((GM)+(off)) = (v))
 #define gr_writel(v,off)	(*(unsigned long*)((GM)+(off)) = (v))
 
+#endif /* alpha */
 
-#endif
+extern void port_out(int value, int port);
+extern void port_outw(int value, int port);
+extern void port_outl(int value, int port);
+extern int port_in(int port);
+extern int port_inw(int port);
+extern int port_inl(int port);
 
 /* Note that the arguments of outb/w are reversed compared with the */
 /* kernel sources. The XFree86 drivers also use this format. */
@@ -347,8 +293,6 @@ extern unsigned char *__svgalib_give_graph_red(void);
 extern unsigned char *__svgalib_give_graph_green(void);
 extern unsigned char *__svgalib_give_graph_blue(void);
  
-#endif
-
 #define zero_sa_mask(maskptr) memset(maskptr, 0, sizeof(sigset_t))
 
 #if 0
@@ -362,3 +306,7 @@ extern unsigned char *__svgalib_give_graph_blue(void);
 #define SVGALIB_RELEASE_SIG SIGPROF
 
 #endif
+
+
+#endif /* _LIBVGA_H */
+
