@@ -30,8 +30,6 @@ This software has NO WARRANTY.  Use it at your own risk.
 #define REAL_MEM_SIZE 	0x10000
 #define REAL_MEM_BLOCKS 	0x100
 
-int __svgalib_lrmi_cpu_type=CPU_386;
-
 struct mem_block
 	{
 	unsigned int size : 20;
@@ -232,7 +230,7 @@ LRMI_init(void)
 	 Map the Interrupt Vectors (0x0 - 0x400) + BIOS data (0x400 - 0x502)
 	 and the ROM (0xa0000 - 0x100000)
 	*/
-	fd_mem = open("/dev/mem", O_RDWR);
+	fd_mem = open("/dev/svgalib_helper", O_RDWR);
 
 	if (fd_mem == -1)
 		{
@@ -260,7 +258,6 @@ LRMI_init(void)
 		return 0;
 		}
 
-
 	/*
 	 Allocate a stack
 	*/
@@ -282,8 +279,7 @@ LRMI_init(void)
 
 	memset(&context.vm, 0, sizeof(context.vm));
 
-	context.vm.cpu_type = __svgalib_lrmi_cpu_type;
-
+        context.vm.cpu_type=CPU_386;
 	/*
 	 Enable kernel emulation of all ints except RETURN_TO_32_INT
 	*/
@@ -516,49 +512,39 @@ em_rep_outs(int size)
 static void
 em_inb(void)
 	{
-	asm volatile ("inb (%w1), %b0"
-	 : "=a" (context.vm.regs.eax)
-	 : "d" (context.vm.regs.edx), "0" (context.vm.regs.eax));
+   	context.vm.regs.eax&=0xffffff00;
+   	context.vm.regs.eax|=port_in(context.vm.regs.edx);
 	}
 
 static void
 em_inw(void)
 	{
-	asm volatile ("inw (%w1), %w0"
-	 : "=a" (context.vm.regs.eax)
-	 : "d" (context.vm.regs.edx), "0" (context.vm.regs.eax));
+   	context.vm.regs.eax&=0xffff0000;
+   	context.vm.regs.eax|=port_inw(context.vm.regs.edx);
 	}
 
 static void
 em_inl(void)
 	{
-	asm volatile ("inl (%w1), %0"
-	 : "=a" (context.vm.regs.eax)
-	 : "d" (context.vm.regs.edx));
+   	context.vm.regs.eax=port_inl(context.vm.regs.edx);
 	}
 
 static void
 em_outb(void)
 	{
-	asm volatile ("outb %b0, (%w1)"
-	 : : "a" (context.vm.regs.eax),
-	 "d" (context.vm.regs.edx));
+   	port_out(context.vm.regs.eax,context.vm.regs.edx);
 	}
 
 static void
 em_outw(void)
 	{
-	asm volatile ("outw %w0, (%w1)"
-	 : : "a" (context.vm.regs.eax),
-	 "d" (context.vm.regs.edx));
+   	port_outw(context.vm.regs.eax,context.vm.regs.edx);
 	}
 
 static void
 em_outl(void)
 	{
-	asm volatile ("outl %0, (%w1)"
-	 : : "a" (context.vm.regs.eax),
-	 "d" (context.vm.regs.edx));
+   	port_outl(context.vm.regs.eax,context.vm.regs.edx);
 	}
 
 static int
