@@ -115,13 +115,10 @@ void mouse_seteventhandler(__mouse_handler handler)
 
 void mouse_close()
 {
-    if(mouse_open) {
-        ms_close();
-        if (currentinthandler != NULL)
-            /* Restore old interrupt. */
-	    sigaction(SIGINT, &oldsiga, NULL);
-    }
-    mouse_open=0;
+    ms_close();
+    if (currentinthandler != NULL)
+	/* Restore old interrupt. */
+	sigaction(SIGINT, &oldsiga, NULL);
 }
 
 
