@@ -103,7 +103,6 @@ static const unsigned char proto[][5] =
     {0xc8, 0x08, 0x00, 0x00, 4},	/* IntelliMouse (PS/2) */
     {0x40, 0x40, 0x40, 0x00, 3},	/* pnp */
     {0x80, 0x80, 0x80, 0x00, 7},	/* Wacom Graphire */
-    {0xc8, 0x08, 0x00, 0x00, 4},	/* DRMOUSE4DS (Digital Research 2-wheel PS/2) */
 };
 
 static void ms_setspeed(const int old, const int new,
@@ -440,8 +439,7 @@ static int ms_init(void)
     if ((__svgalib_mouse_fd = open(m_dev, O_RDWR | O_NDELAY )) < 0)
 	return -1;
 
-    if (m_type == MOUSE_BUSMOUSE || m_type == MOUSE_PS2 
-    		|| m_type == MOUSE_IMPS2 || m_type == MOUSE_GPM || m_type == MOUSE_DRMOUSE4DS)
+    if (m_type == MOUSE_BUSMOUSE || m_type == MOUSE_PS2 || m_type == MOUSE_IMPS2 || m_type == MOUSE_GPM)
 	m_modem_ctl = 0;
 
     /* If no signal will change there is no need to restore
@@ -499,7 +497,7 @@ static int ms_init(void)
     	usleep(75000);
     	/* Start sending coordinates */
     	write(__svgalib_mouse_fd, "\rST\r", 4);
-    } else if (m_type == MOUSE_IMPS2 || m_type == MOUSE_DRMOUSE4DS) {
+    } else if (m_type == MOUSE_IMPS2) {
 	/* Initialize the mouse into wheel mode */
 	write(__svgalib_mouse_fd, "\363\310\363\144\363\120", 6);
     } else if (m_type == MOUSE_PNP) {
@@ -560,14 +558,7 @@ static int ms_init(void)
             MOUSE_RESETBUTTON;
         mouse_caps.axes = MOUSE_6DIM;
         break;
-    
-    case MOUSE_DRMOUSE4DS:
-    	/* X, Y, RX and RY (two wheels), 3 buttons */
-	mouse_caps.axes = MOUSE_2DIM | MOUSE_RXDIM | MOUSE_RYDIM;
-	mouse_caps.buttons = MOUSE_LEFTBUTTON | MOUSE_RIGHTBUTTON | MOUSE_MIDDLEBUTTON;
-	mouse_caps.info = MOUSE_INFO_WHEEL;
-	break;
-	
+        
     case MOUSE_INTELLIMOUSE:
     case MOUSE_IMPS2:
     case MOUSE_WACOM_GRAPHIRE:
@@ -750,8 +741,7 @@ static int get_ms_event(int wait) {
 	goto handle_packets;
     }
     /* Check whether it's a valid data packet. */
-    if ((m_type != MOUSE_PS2)&&(m_type != MOUSE_IMPS2)&&(m_type != MOUSE_SPACEBALL)
-    	&&(m_type != MOUSE_WACOM_GRAPHIRE) && (m_type != MOUSE_DRMOUSE4DS) )
+    if ((m_type != MOUSE_PS2)&&(m_type != MOUSE_IMPS2)&&(m_type != MOUSE_SPACEBALL)&&(m_type != MOUSE_WACOM_GRAPHIRE))
 	for (j = 1; j < proto[m_type][4]; j++)
 	    if ((buf[i + j] & proto[m_type][2]) != proto[m_type][3]
 		|| buf[i + j] == 0x80) {
@@ -907,58 +897,6 @@ static int get_ms_event(int wait) {
 	dx = (buf[i] & 0x10) ? buf[i + 1] - 256 : buf[i + 1];
 	dy = (buf[i] & 0x20) ? -(buf[i + 2] - 256) : -buf[i + 2];
 	break;
-    case MOUSE_DRMOUSE4DS:
-    /* Digital Research 4-Axis mouse - like the PS/2 IntelliMouse, but 
-       has two wheels.  */
-        /* This bit modified from the gpm 1.13 imps2 patch by Tim Goodwin */
-        but = ((buf[i] & 1) << 2) /* left */
-            | ((buf[i] & 6) >> 1); /* middle and right */
-        dx = (buf[i] & 0x10) ? buf[i + 1] - 256 : buf[i + 1];
-        dy = (buf[i] & 0x20) ? -(buf[i + 2] - 256) : -buf[i + 2];
-
-        /* Did we turn the wheel? */
-        if((wheel = buf[i + 3]) != 0) {
-#ifdef DEBUG_WHEEL
-            fprintf(stderr, " Wheel turned (0x%02x)", wheel);
-#endif
-	/* RX reports as 1 or F, RY reports as 2 or E */
-	/* both never report at the same time, which makes life easier */
-	if (m_wheel_delta) 
-	{
-		switch (wheel & 0x0F)
-		{
-		case 1:		drx = m_wheel_delta; break;
-		case 15:	drx = -m_wheel_delta; break;
-		case 2:		dry = -m_wheel_delta; break;
-		case 14:	dry = m_wheel_delta; break;
-		};
-#ifdef DEBUG_WHEEL
-                fprintf(stderr, "; RX axis delta = %d : RY axis delta = %d", drx, dry);
-#endif
-
-            /* Are fake keyboard events enabled? */
-            if(m_fake_kbd_events && __svgalib_keyboard_eventhandler) {
-                /* Fake keypresses... */
-                int key = ((drx+dry < 0) ? m_fake_upscancode : m_fake_downscancode);
-#ifdef DEBUG_WHEEL
-                fprintf(stderr, "; pressing scancode %d", key);
-#endif
-                 __svgalib_keyboard_eventhandler(key, 1); /* press */
-                /* todo: delay */
-#ifdef DEBUG_WHEEL
-                fprintf(stderr, ", releasing scancode %d", key);
-#endif
-                __svgalib_keyboard_eventhandler(key, 0); /* release */
-            }
-#ifdef DEBUG_WHEEL
-            fprintf(stderr, ".\n");
-#endif
-
-
-	}
-    }
-    break;
-    
     case MOUSE_IMPS2:           /* PS/2 IntelliMouse */
         /* This bit modified from the gpm 1.13 imps2 patch by Tim Goodwin */
         but = ((buf[i] & 1) << 2) /* left */
