@@ -152,6 +152,8 @@ static int mx_modeavailable(int mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
+    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
+
     if ((mode < G640x480x256 )
 	|| mode == G720x348x2)
 	return __svgalib_vga_driverspecs.modeavailable(mode);
@@ -494,7 +496,6 @@ static int mx_init(int force, int par1, int par2)
 {
    unsigned long buf[64];
    int found=0;
-   int _ioperm=0;
 
     mx_unlock();
     if (force) {
@@ -505,15 +506,7 @@ static int mx_init(int force, int par1, int par2)
     };
     if(mx_memory==0)mx_memory=1024<<((__svgalib_inCR(0x57)>>3)& 3);
 
-    if (getenv("IOPERM") == NULL) {
-      _ioperm=1;
-      if (iopl(3) < 0) {
-        printf("svgalib: mx: cannot get I/O permissions\n");
-        exit(1);
-      }
-    }
     found=__svgalib_pci_find_vendor_vga(0x10d9,buf,0);
-    if (_ioperm) iopl(0);
     mx_linear_base=0;
     if (!found){
        mx_linear_base=buf[4]&0xffffff00;
@@ -552,7 +545,7 @@ static int mx_init(int force, int par1, int par2)
     cardspecs->maxPixelClock24bpp = 160000;
     cardspecs->maxPixelClock32bpp = 160000;
     cardspecs->flags = INTERLACE_DIVIDE_VERT | CLOCK_PROGRAMMABLE;
-    cardspecs->maxHorizontalCrtc = 4088;
+    cardspecs->maxHorizontalCrtc = 2040;
     cardspecs->maxPixelClock4bpp = 0;
     cardspecs->nClocks =0;
     cardspecs->mapClock = mx_map_clock;
