@@ -1,5 +1,5 @@
 /*
-Riva 128 driver - Matan Ziv-Av zivav@cs.bgu.ac.il
+Riva 128 driver - Matan Ziv-Av matan@svgalib.org
 please report problems to me,
 
 This driver is based on the XFREE86 nv3 driver, developed by
@@ -50,6 +50,11 @@ there are still the following problems:
 static volatile unsigned  *nvPFBPort;
 static volatile unsigned  *nvPRAMDACPort;
 static volatile unsigned  *nvPEXTDEVPort;
+static volatile unsigned  *nvPVGA0Port;
+static volatile unsigned  *nvPVGA1Port;
+static volatile unsigned  *nvPVGA2Port;
+
+#include "nv3io.c"
 
 static int nv3_init(int, int, int);
 static void nv3_unlock(void);
@@ -61,7 +66,7 @@ static int nv3_is_linear;
 static CardSpecs *cardspecs;
 static int PLL_INPUT_FREQ;
 
-static unsigned long MMIOBASE=0, LINEARBASE;
+static unsigned long MMIOBASE, LINEARBASE;
 
 enum {
    Riva128 = 0, RivaTNT, GEFORCE
@@ -105,15 +110,16 @@ static int nv3_saveregs(unsigned char regs[])
     int i;
     nv3_unlock();		/* May be locked again by other programs (e.g. X) */
 
-    regs[NV3REG_SAVE(0)] = PCRTC_Read(REPAINT0);
-    regs[NV3REG_SAVE(1)] = PCRTC_Read(REPAINT1);
-    regs[NV3REG_SAVE(2)] = PCRTC_Read(EXTRA);
-    regs[NV3REG_SAVE(3)] = PCRTC_Read(PIXEL);
-    regs[NV3REG_SAVE(4)] = PCRTC_Read(HORIZ_EXTRA);
-    regs[NV3REG_SAVE(5)] = PCRTC_Read(FIFO_CONTROL);
-    regs[NV3REG_SAVE(6)] = PCRTC_Read(FIFO);
+    regs[NV3REG_SAVE(0)] = __svgalib_incrtc(NV_PCRTC_REPAINT0);
+    regs[NV3REG_SAVE(1)] = __svgalib_incrtc(NV_PCRTC_REPAINT1);
+    regs[NV3REG_SAVE(2)] = __svgalib_incrtc(NV_PCRTC_EXTRA);
+    regs[NV3REG_SAVE(3)] = __svgalib_incrtc(NV_PCRTC_PIXEL);
+    regs[NV3REG_SAVE(4)] = __svgalib_incrtc(NV_PCRTC_HORIZ_EXTRA);
+    regs[NV3REG_SAVE(5)] = __svgalib_incrtc(NV_PCRTC_FIFO_CONTROL);
+    regs[NV3REG_SAVE(6)] = __svgalib_incrtc(NV_PCRTC_FIFO);
     regs[NV3REG_SAVE(7)] = 0;
 
+
    k = PFB_Read(CONFIG_0) ;
    regs[NV3REG_SAVE(8)] = k&0xff;
    regs[NV3REG_SAVE(9)] = (k >> 8)&0xff;
@@ -146,28 +152,28 @@ static void nv3_setregs(const unsigned char regs[], int mode)
 {   unsigned long k ; 
     nv3_unlock();		/* May be locked again by other programs (eg. X) */
 
-   PCRTC_Write(REPAINT0,regs[NV3REG_SAVE(0)]);
-   PCRTC_Write(REPAINT1,regs[NV3REG_SAVE(1)]);
-   PCRTC_Write(EXTRA,regs[NV3REG_SAVE(2)]);
-   PCRTC_Write(PIXEL,regs[NV3REG_SAVE(3)]);
-   PCRTC_Write(HORIZ_EXTRA,regs[NV3REG_SAVE(4)]); 
-   PCRTC_Write(FIFO_CONTROL,regs[NV3REG_SAVE(5)]); 
+   __svgalib_outcrtc(NV_PCRTC_REPAINT0,regs[NV3REG_SAVE(0)]);
+   __svgalib_outcrtc(NV_PCRTC_REPAINT1,regs[NV3REG_SAVE(1)]);
+   __svgalib_outcrtc(NV_PCRTC_EXTRA,regs[NV3REG_SAVE(2)]);
+   __svgalib_outcrtc(NV_PCRTC_PIXEL,regs[NV3REG_SAVE(3)]);
+   __svgalib_outcrtc(NV_PCRTC_HORIZ_EXTRA,regs[NV3REG_SAVE(4)]); 
+   __svgalib_outcrtc(NV_PCRTC_FIFO_CONTROL,regs[NV3REG_SAVE(5)]); 
 /*  PCRTC_Write(FIFO,regs[NV3REG_SAVE(6)]); */
 
    __svgalib_outcrtc(0x1c,regs[88]); /* this enables banking at 0xa0000 */
-   __svgalib_outcrtc(0x1d,regs[89]);
-   __svgalib_outcrtc(0x1e,regs[90]);
+   __svgalib_outcrtc(0x1d,regs[89]); 
+   __svgalib_outcrtc(0x1e,regs[90]); 
    __svgalib_outcrtc(0x31,0); /* disable cursor */
 
    k=regs[NV3REG_SAVE(8)]+(regs[NV3REG_SAVE(9)] << 8)+(regs[NV3REG_SAVE(10)] << 16)+(regs[NV3REG_SAVE(11)] << 24); 
-          PFB_Write(CONFIG_0,k);
+   PFB_Write(CONFIG_0,k);
 
    k=regs[NV3REG_SAVE(12)]+(regs[NV3REG_SAVE(13)] << 8)+(regs[NV3REG_SAVE(14)] << 16)+(regs[NV3REG_SAVE(15)] << 24);
-          PRAMDAC_Write(VPLL_COEFF,k);
+   PRAMDAC_Write(VPLL_COEFF,k);
    k=regs[NV3REG_SAVE(16)]+(regs[NV3REG_SAVE(17)] << 8)+(regs[NV3REG_SAVE(18)] << 16)+(regs[NV3REG_SAVE(19)] << 24);
-          PRAMDAC_Write(PLL_COEFF_SELECT,k);
+   PRAMDAC_Write(PLL_COEFF_SELECT,k);
    k=regs[NV3REG_SAVE(20)]+(regs[NV3REG_SAVE(21)] << 8)+(regs[NV3REG_SAVE(22)] << 16)+(regs[NV3REG_SAVE(23)] << 24);
-          PRAMDAC_Write(GENERAL_CONTROL,k);
+   PRAMDAC_Write(GENERAL_CONTROL,k);
 }
 
 
@@ -178,6 +184,10 @@ static int nv3_modeavailable(int mode)
     struct info *info;
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
+
+    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
+
+    if(modeinfo->bitsPerPixel==24)return 0;
     
     if ((mode < G640x480x256 )
 	|| mode == G720x348x2)
@@ -189,11 +199,8 @@ static int nv3_modeavailable(int mode)
     
     modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
    
-    if(modeinfo->bitsPerPixel==24) {
-	free(modeinfo);
-        return 0;
-    }
-    
+/*    if(modeinfo->greenWeight==5)return 0; */
+
     modetiming = malloc(sizeof(ModeTiming));
     if (__svgalib_getmodetiming(modetiming, modeinfo, cardspecs)) {
 	free(modetiming);
@@ -206,7 +213,6 @@ static int nv3_modeavailable(int mode)
     return SVGADRV;
 }
 
-
 static int NV3ClockSelect(float clockIn,float *clockOut,int *mOut,
                                         int *nOut,int *pOut);
 
@@ -292,9 +298,9 @@ static void nv3_initializemode(unsigned char *moderegs,
    moderegs[NV3REG_SAVE(14)] = (k >> 16) & 0xff ;
    moderegs[NV3REG_SAVE(15)] = ( k >> 24 ) & 0xff ;
 
-  CalculateCRTC(modetiming,modeinfo,moderegs);
+   CalculateCRTC(modetiming,modeinfo,moderegs);
 
-  moderegs[NV3REG_SAVE(1)]=
+   moderegs[NV3REG_SAVE(1)]=
     PCRTC_Val(REPAINT1_LARGE_SCREEN,modetiming->CrtcHDisplay<1280) |
     PCRTC_Def(REPAINT1_PALETTE_WIDTH,6BITS);
 
@@ -328,15 +334,15 @@ static void nv3_initializemode(unsigned char *moderegs,
    switch(nv3_chiptype){
       case Riva128:
           k=PRAMDAC_Def(PLL_COEFF_SELECT_MPLL_SOURCE,PROG)|
-                            PRAMDAC_Def(PLL_COEFF_SELECT_VPLL_SOURCE,PROG)|
-                            PRAMDAC_Def(PLL_COEFF_SELECT_VCLK_RATIO,DB2);
+            PRAMDAC_Def(PLL_COEFF_SELECT_VPLL_SOURCE,PROG)|
+            PRAMDAC_Def(PLL_COEFF_SELECT_VCLK_RATIO,DB2);
+          k=0x10010100;
           break;
       case RivaTNT:
       case GEFORCE:
           k=0x10000700;
           break;
    };
-   
    moderegs[NV3REG_SAVE(16)] = k&0xff;
    moderegs[NV3REG_SAVE(17)] = (k >> 8)&0xff;
    moderegs[NV3REG_SAVE(18)] = (k >> 16) & 0xff ;
@@ -346,16 +352,15 @@ static void nv3_initializemode(unsigned char *moderegs,
    * give some significant performance gains. Will perhaps experiment 
    * later on with this stuff!  
    */
-   if(nv3_chiptype<GEFORCE) {
-       k=
-           PFB_Val(CONFIG_0_RESOLUTION,((modeinfo->lineWidth+31)/32))|
-           PFB_Val(CONFIG_0_PIXEL_DEPTH,pixelDepth)|
-           PFB_Def(CONFIG_0_TILING,ENABLED); 
-       moderegs[NV3REG_SAVE(8)] = k&0xff;
-       moderegs[NV3REG_SAVE(9)] = (k >> 8)&0xff;
-       moderegs[NV3REG_SAVE(10)] = (k >> 16) & 0xff ;
-       moderegs[NV3REG_SAVE(11)] = ( k >> 24 ) & 0xff ;
-   }
+   k=
+      PFB_Val(CONFIG_0_RESOLUTION,((modeinfo->lineWidth+31)/32))|
+      PFB_Val(CONFIG_0_PIXEL_DEPTH,pixelDepth)|
+      PFB_Def(CONFIG_0_TILING,ENABLED); 
+   moderegs[NV3REG_SAVE(8)] = k&0xff;
+   moderegs[NV3REG_SAVE(9)] = (k >> 8)&0xff;
+   moderegs[NV3REG_SAVE(10)] = (k >> 16) & 0xff ;
+   moderegs[NV3REG_SAVE(11)] = ( k >> 24 ) & 0xff ;
+
   moderegs[88]=28;
 
   if(modeinfo->greenWeight==5){moderegs[NV3REG_SAVE(21)]&=0xef;}; 
@@ -374,28 +379,9 @@ static int nv3_setmode(int mode, int prv_mode)
     int i;
     if ((mode < G640x480x256 /*&& mode != G320x200x256*/)
 	|| mode == G720x348x2) {
-
-	unsigned int k;
-
         if(nv3_chiptype==Riva128)
             PRAMDAC_Write(PLL_COEFF_SELECT,0x00000100);
             else PRAMDAC_Write(PLL_COEFF_SELECT,0x00000500);
-
-        __svgalib_outcrtc(NV_PCRTC_REPAINT0,0);
-        __svgalib_outcrtc(NV_PCRTC_REPAINT1,0x3d);
-        __svgalib_outcrtc(NV_PCRTC_EXTRA,0);
-        __svgalib_outcrtc(NV_PCRTC_PIXEL,0);
-        __svgalib_outcrtc(NV_PCRTC_HORIZ_EXTRA,0);
-        __svgalib_outcrtc(NV_PCRTC_FIFO_CONTROL,0x83);
-        __svgalib_outcrtc(0x1c,0x18);
-        __svgalib_outcrtc(0x1d,0);
-        __svgalib_outcrtc(0x1e,0);
-        __svgalib_outcrtc(0x30,0);
-        __svgalib_outcrtc(0x31,0);
-        k =  PRAMDAC_Read(GENERAL_CONTROL);
-        k &= ~0x00100000;
-        PRAMDAC_Write(GENERAL_CONTROL,k);
-
 	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
     }
 
@@ -432,14 +418,14 @@ static int nv3_setmode(int mode, int prv_mode)
 
 static void nv3_unlock(void)
 {
-    __svgalib_outcrtc(0x11,__svgalib_incrtc(0x11)&0x7f);
+//    __svgalib_outcrtc(0x11,__svgalib_incrtc(0x11)&0x7f);
     __svgalib_outseq(LOCK_EXT_INDEX,UNLOCK_EXT_MAGIC);    
 }
 
 static void nv4_unlock(void)
 {
-    __svgalib_outcrtc(0x11,__svgalib_incrtc(0x11)&0x7f);
-    __svgalib_outcrtc(0x1f, UNLOCK_EXT_MAGIC);
+    __svgalib_nv3_outcrtc(0x11,__svgalib_incrtc(0x11)&0x7f);
+    __svgalib_nv3_outcrtc(0x1f, UNLOCK_EXT_MAGIC);
 }
 
 
@@ -448,7 +434,7 @@ static void nv4_unlock(void)
 
 static void nv3_lock(void)
 {
-__svgalib_outseq(LOCK_EXT_INDEX,UNLOCK_EXT_MAGIC+1);    
+    __svgalib_outseq(LOCK_EXT_INDEX,UNLOCK_EXT_MAGIC+1);    
 
 }
 
@@ -457,43 +443,38 @@ __svgalib_outseq(LOCK_EXT_INDEX,UNLOCK_EXT_MAGIC+1);
 static int nv3_test(void)
 {  unsigned long buf[64];
    int found=0;
-   int _ioperm=0;
 
-   if (getenv("IOPERM") == NULL) {
-     _ioperm=1;
-     if (iopl(3) < 0) {
-       printf("svgalib: nv3: cannot get I/O permissions\n");
-       exit(1);
-     }
-   }
    found=__svgalib_pci_find_vendor_vga(0x12d2,buf,0);
    if (found) {
       found=__svgalib_pci_find_vendor_vga(0x10de,buf,0);
-      if (_ioperm) iopl(0);
       if(found)return 0;
    };
-   if (_ioperm) iopl(0);
+   if ((((buf[0]>>16)&0xffff)!=0x18)&&
+       (((buf[0]>>16)&0xffff)!=0x20)&&
+       (((buf[0]>>16)&0xffff)!=0x29)&&
+       (((buf[0]>>16)&0xffff)!=0x2C)&&
+       (((buf[0]>>16)&0xffff)!=0x2D)&&
+       (((buf[0]>>16)&0xffff)!=0xA0)&&
+       (((buf[0]>>16)&0xffff)!=0x28)&&
+       (((buf[0]>>16)&0xffff)!=0x100)&&
+       (((buf[0]>>16)&0xffff)!=0x101)&&
+       (((buf[0]>>16)&0xffff)!=0x103)
+       )return 0;
    switch(buf[0]>>16){
       case 0x18: nv3_chiptype=Riva128; break;
       case 0x20:
-      case 0x29: case 0x2A: case 0x2B: case 0x2C: case 0x2D: case 0x2E: case 0x2F:
-      case 0xA0:
+      case 0x29: case 0x2C: case 0x2D: case 0xA0:
       case 0x28: nv3_chiptype=RivaTNT; break;
-      case 0x150: case 0x151: case 0x152: case 0x153:
-      case 0x110: case 0x111: case 0x113:
       case 0x101: case 0x103:
       case 0x100: nv3_chiptype=GEFORCE; break;
-      default: nv3_chiptype=-1;
+      default: nv3_chiptype=Riva128;
    };
-
-   if(nv3_chiptype==-1)return 0;
-   MMIOBASE=0;
-   LINEARBASE=0;
+   MMIOBASE=buf[4]&0xffffff00;
+   LINEARBASE=buf[5]&0xffffff00;
    nv3_init(0,0,0);
-return 1;
+   return 1;
 }
 
-
 /* No r/w paging - I guess it's possible, but is it useful? */
 static void nv3_setrdpage(int page)
 {
@@ -510,8 +491,8 @@ static void nv3_setdisplaystart(int address)
   address=address >> 2;
   __svgalib_outcrtc(0x0d,address&0xff);
   __svgalib_outcrtc(0x0c,(address>>8)&0xff);
-  byte=PCRTC_Read(REPAINT0) & 0xe0;
-  PCRTC_Write(REPAINT0,((address>>16)&0x1f)|byte);
+  byte=__svgalib_incrtc(NV_PCRTC_REPAINT0) & 0xe0;
+  __svgalib_outcrtc(NV_PCRTC_REPAINT0,((address>>16)&0x1f)|byte);
   
 }
 
@@ -522,8 +503,8 @@ static void nv3_setlogicalwidth(int width)
 {  int byte ;
 
   __svgalib_outcrtc(0x13,(width >> 3)&0xff);
-  byte=PCRTC_Read(REPAINT0) & 0x1f;
-  PCRTC_Write(REPAINT0,SetBitField(width,13:11,7:5)|byte);
+  byte=__svgalib_incrtc(NV_PCRTC_REPAINT0) & 0x1f;
+  __svgalib_outcrtc(NV_PCRTC_REPAINT0,SetBitField(width,13:11,7:5)|byte);
 
 }
 
@@ -578,7 +559,7 @@ DriverSpecs __svgalib_nv3_driverspecs =
 
 #define MapDevice(device,base) \
   nv##device##Port=(unsigned*)(mmap(0, \
-     	DEVICE_SIZE(device),PROT_WRITE,MAP_SHARED,__svgalib_mem_fd,\
+     	DEVICE_SIZE(device),PROT_READ|PROT_WRITE,MAP_SHARED,__svgalib_mem_fd,\
         (MMIOBASE)+DEVICE_BASE(device)))
 
 /* Initialize chipset (called after detection) */
@@ -589,34 +570,20 @@ static int nv3_init(int force, int par1, int par2)
                             "nv4, RivaTNT/RivaTNT2", 
                             "nv10, GeForce"};
     
-    nv3_unlock();
-    
     if(MMIOBASE==0) {
        unsigned long buf[64];
-       int _ioperm=0,found;
-       
-       if (getenv("IOPERM") == NULL) {
-         _ioperm=1;
-         if (iopl(3) < 0) {
-           printf("svgalib: nv3: cannot get I/O permissions\n");
-           exit(1);
-         }
-       }
+       int found;
        
        found=__svgalib_pci_find_vendor_vga(0x12d2,buf,0);
        if (found) {
           found=__svgalib_pci_find_vendor_vga(0x10de,buf,0);
-          if (_ioperm) iopl(0);
           if(found)return 0;
-       } else if (_ioperm) iopl(0);
+       } 
        switch(buf[0]>>16){
           case 0x18: nv3_chiptype=Riva128; break;
           case 0x20:
-          case 0x29: case 0x2A: case 0x2B: case 0x2C: case 0x2D: case 0x2E: case 0x2F:
-          case 0xA0:
+          case 0x29: case 0x2C: case 0x2D: case 0xA0:
           case 0x28: nv3_chiptype=RivaTNT; break;
-          case 0x150: case 0x151: case 0x152: case 0x153:
-          case 0x110: case 0x111: case 0x113:
           case 0x101: case 0x103:
           case 0x100: nv3_chiptype=GEFORCE; break;
           default: nv3_chiptype=Riva128;
@@ -631,7 +598,9 @@ static int nv3_init(int force, int par1, int par2)
     MapDevice(PRAMDAC,regBase);
     MapDevice(PFB,regBase);
     MapDevice(PEXTDEV,regBase);
-     
+    MapDevice(PVGA0,regBase);
+    MapDevice(PVGA1,regBase);
+    MapDevice(PVGA2,regBase);
     if(!force){
        int boot0;
        
@@ -646,40 +615,50 @@ static int nv3_init(int force, int par1, int par2)
                  if(nv3_memory==2048)nv3_memory=32768;
                  break;
           case GEFORCE:
-             	 nv3_memory=16384; /* do this later */
+             	 nv3_memory=8192; /* do this later */
                  break;
-       };
-    };
-
+       }
+    }
+/*    if((nv3_chiptype==RivaTNT)||(nv3_chiptype==GEFORCE)) */
+        nv3_mapio();
     {
        int temp;
        
        temp=PEXTDEV_Read(0);
        switch(nv3_chiptype){
           case Riva128:
+             nv3_unlock();
              PLL_INPUT_FREQ= (temp&0x20) ? 14318 : 13500;
              break;
           case RivaTNT:
           case GEFORCE:
+             nv4_unlock();
              PLL_INPUT_FREQ= (temp&0x40) ? 14318 : 13500;
              break;
        };
        
     };
-
     if (__svgalib_driver_report) {
 	printf("Using RIVA driver, %iKB, Type:%s.\n",nv3_memory,architectures[nv3_chiptype]);
     };
 
     cardspecs = malloc(sizeof(CardSpecs));
     cardspecs->videoMemory = nv3_memory;
-    cardspecs->maxPixelClock4bpp = 75000;
-    cardspecs->maxPixelClock8bpp = 230000;
-    cardspecs->maxPixelClock16bpp = 230000;
-    cardspecs->maxPixelClock24bpp = 0;
-    cardspecs->maxPixelClock32bpp = 230000;
+    if(nv3_chiptype==Riva128) {
+       cardspecs->maxPixelClock4bpp = 75000;	
+       cardspecs->maxPixelClock8bpp = 230000;	
+       cardspecs->maxPixelClock16bpp = 230000;	
+       cardspecs->maxPixelClock24bpp = 0;
+       cardspecs->maxPixelClock32bpp = 230000;
+    } else {
+       cardspecs->maxPixelClock4bpp = 75000;	
+       cardspecs->maxPixelClock8bpp = 350000;	
+       cardspecs->maxPixelClock16bpp = 350000;	
+       cardspecs->maxPixelClock24bpp = 0;
+       cardspecs->maxPixelClock32bpp = 350000;    
+    }
     cardspecs->flags = CLOCK_PROGRAMMABLE ;
-    cardspecs->maxHorizontalCrtc = 4088;
+    cardspecs->maxHorizontalCrtc = 4080;
     cardspecs->maxPixelClock4bpp = 0;
     cardspecs->nClocks =0;
     cardspecs->clocks = NULL;
@@ -695,6 +674,7 @@ static int nv3_init(int force, int par1, int par2)
     __svgalib_banked_mem_size=0x10000;
     __svgalib_linear_mem_base=LINEARBASE;
     __svgalib_linear_mem_size=nv3_memory*0x400;
+
     return 0;
 }
 
@@ -707,7 +687,7 @@ static int NV3ClockSelect(float clockIn,float *clockOut,int *mOut,
   float best=0.0;
   float diff;
   int nMax,nMin;
-  
+
   *clockOut=0.0;
   for(p=P_MIN;p<=P_MAX;p++) {
     for(m=M_MIN;m<=M_MAX;m++) {
@@ -732,4 +712,3 @@ static int NV3ClockSelect(float clockIn,float *clockOut,int *mOut,
   return (best!=0.0);    
 }
 
-
