@@ -1,9 +1,9 @@
 /*
 ATI Rage chipset 
 
-By Matan Ziv-Av (zivav@cs.bgu.ac.il)
+By Matan Ziv-Av (matan@svgalib.org)
 
-Alpha release - Please report to me any problem with the driver 
+Please report to me any problem with the driver 
 
 It works on my Xpert98 AGP 8MB SDRAM.
 
@@ -41,333 +41,12 @@ Copyright 1994 through 1998 by Marc Aurele La France (TSI @ UQV), tsi@ualberta.c
 #include "interface.h"
 #include "accel.h"
 #include "vgapci.h"
+#include "rage.h"
 
 /*
 #define RAGE_TOTAL_REGS (VGA_TOTAL_REGS + 64)
 */
-#define RAGE_TOTAL_REGS (VGA_TOTAL_REGS + 1400)
-
-#define SPARSE_IO 0
-#define BLOCK_IO 1
-
-#define _ByteMask(__Byte)	((unsigned char)(-1) << (8 * (__Byte)))
-#define GetByte(_Value, _Byte)	GetBits(_Value, _ByteMask(_Byte))
-
-#define GetReg(_Register, _Index)                               \
-        (                                                       \
-                outb(_Register, _Index),                        \
-                inb(_Register + 1)                              \
-        )
-#define PutReg(_Register, _Index, _Value)                       \
-        outw(_Register, ((_Value) << 8) | (_Index))
-
-#define ATIIOPort(_PortTag)							\
-        (((ATIIODecoding == SPARSE_IO) ?					\
-          (((_PortTag) & SPARSE_IO_SELECT) | ((_PortTag) & IO_BYTE_SELECT)) :	\
-          (((_PortTag) & BLOCK_IO_SELECT)  | ((_PortTag) & IO_BYTE_SELECT))) |	\
-         ATIIOBase)
-
-#define ATTRX			0x03c0u
-#define ATTRD			0x03c1u
-#define SEQX			0x03c4u
-#define SEQD			0x03c5u
-#define GRAX			0x03ceu
-#define GRAD			0x03cfu
-#define GENS1			0x03dau
-#define CRTX			0x03d4u
-#define CRTD			0x03d5u
-
-#define SPARSE_IO_BASE		0x03fcu
-#define SPARSE_IO_SELECT	0xfc00u
-#define BLOCK_IO_BASE		0xff00u
-#define BLOCK_IO_SELECT		0x00fcu
-#define IO_BYTE_SELECT		0x0003u
-
-#define _UnitOf(___Value)	((((___Value) ^ ((___Value) - 1)) + 1) >> 1)
-#define GetBits(__Value, _Mask)	(((__Value) & (_Mask)) / _UnitOf(_Mask))
-#define SetBits(__Value, _Mask)	(((__Value) * _UnitOf(_Mask)) & (_Mask))
-#define IOPortTag(_SparseIOSelect, _BlockIOSelect)	\
-	(SetBits(_SparseIOSelect, SPARSE_IO_SELECT) | \
-	 SetBits(_BlockIOSelect, BLOCK_IO_SELECT))
-#define SparseIOTag(_IOSelect)	IOPortTag(_IOSelect, (unsigned)(-1))
-#define BlockIOTag(_IOSelect)	IOPortTag((unsigned)(-1), _IOSelect)
-
-#define CRTC_H_TOTAL_DISP	IOPortTag(0x00u, 0x00u)
-#define CRTC_H_SYNC_STRT_WID	IOPortTag(0x01u, 0x01u)
-#define CRTC_V_TOTAL_DISP	IOPortTag(0x02u, 0x02u)
-#define CRTC_V_SYNC_STRT_WID	IOPortTag(0x03u, 0x03u)
-#define CRTC_VLINE_CRNT_VLINE	IOPortTag(0x04u, 0x04u)
-#define CRTC_OFF_PITCH		IOPortTag(0x05u, 0x05u)
-
-#define SCRATCH_REG0		IOPortTag(0x10u, 0x20u)
-#define OVR_WID_LEFT_RIGHT	IOPortTag(0x09u, 0x11u)
-#define OVR_WID_TOP_BOTTOM	IOPortTag(0x0au, 0x12u)
-#define CLOCK_CNTL		IOPortTag(0x12u, 0x24u)
-#define BUS_CNTL		IOPortTag(0x13u, 0x28u)
-#define MEM_VGA_WP_SEL		IOPortTag(0x15u, 0x2du)
-#define MEM_VGA_RP_SEL		IOPortTag(0x16u, 0x2eu)
-#define DAC_CNTL		IOPortTag(0x18u, 0x31u)
-#define DAC_REGS		IOPortTag(0x17u, 0x30u)	/* 4 separate bytes */
-#define CRTC_GEN_CNTL		IOPortTag(0x07u, 0x07u)
-#define DSP_CONFIG		BlockIOTag(0x08u)	/* VTB/GTB/LT */
-#define DSP_ON_OFF		BlockIOTag(0x09u)	/* VTB/GTB/LT */
-#define OVR_CLR			IOPortTag(0x08u, 0x10u)
-#define MEM_INFO		IOPortTag(0x14u, 0x2cu) /* Renamed MEM_CNTL */
-#define CONFIG_CNTL		IOPortTag(0x1au, 0x37u)
-#define GEN_TEST_CNTL		IOPortTag(0x19u, 0x34u)
-#define CRTC_INT_CNTL		IOPortTag(0x06u, 0x06u)
-#define CONFIG_STATUS64_0	IOPortTag(0x1cu, 0x39u)
-
-#define MEM_BUF_CNTL		BlockIOTag(0x0bu)	/* VTB/GTB/LT */
-#define SHARED_CNTL		BlockIOTag(0x0cu)	/* VTB/GTB/LT */
-#define SHARED_MEM_CONFIG	BlockIOTag(0x0du)	/* GT3 */
-
-#define CONFIG_CHIP_ID		IOPortTag(0x1bu, 0x38u) /* Read */
-#define CFG_CHIP_TYPE0			0x000000fful
-#define CFG_CHIP_TYPE1			0x0000ff00ul
-#define CFG_CHIP_TYPE			0x0000fffful
-#define CFG_CHIP_CLASS			0x00ff0000ul
-#define CFG_CHIP_REV			0xff000000ul
-#define CFG_CHIP_VERSION		0x07000000ul	/* 264xT */
-#define CFG_CHIP_FOUNDRY		0x38000000ul	/* 264xT */
-#define CFG_CHIP_REVISION		0xc0000000ul	/* 264xT */
-#define CFG_MEM_TYPE_T			0x00000007ul
-#define CFG_MEM_TYPE			0x00000038ul
-
-#define DAC_8BIT_EN			0x00000100ul
-
-#define CRTC_OFFSET			0x000ffffful
-#define CRTC_H_TOTAL			0x000001fful
-#define CRTC_H_DISP			0x01ff0000ul
-#define CRTC_H_SYNC_STRT		0x000000fful
-#define CRTC_H_SYNC_DLY			0x00000700ul
-#define CRTC_H_SYNC_POL			0x00200000ul
-#define CRTC_H_SYNC_WID			0x001f0000ul
-#define CRTC_H_SYNC_STRT_HI		0x00001000ul
-#define CRTC_V_TOTAL			0x000007fful
-#define CRTC_V_DISP			0x07ff0000ul
-#define CRTC_V_SYNC_STRT		0x000007fful
-#define CRTC_V_SYNC_WID			0x001f0000ul
-#define CRTC_V_SYNC_POL			0x00200000ul
-#define CRTC_PIX_WIDTH			0x00000700ul
-#define CRTC_PIX_WIDTH_1BPP			0x00000000ul
-#define CRTC_PIX_WIDTH_4BPP			0x00000100ul
-#define CRTC_PIX_WIDTH_8BPP			0x00000200ul
-#define CRTC_PIX_WIDTH_15BPP			0x00000300ul
-#define CRTC_PIX_WIDTH_16BPP			0x00000400ul
-#define CRTC_PIX_WIDTH_24BPP			0x00000500ul
-#define CRTC_PIX_WIDTH_32BPP			0x00000600ul
-#define CRTC_VBLANK			0x00000001ul
-#define CRTC_VBLANK_INT_EN		0x00000002ul
-#define CRTC_VBLANK_INT			0x00000004ul
-#define CRTC_VLINE_INT_EN		0x00000008ul
-#define CRTC_VLINE_INT			0x00000010ul
-#define CRTC_VLINE_SYNC			0x00000020ul
-#define CRTC_FRAME			0x00000040ul
-/*	?				0x0000ff80ul */
-#define CRTC_SNAPSHOT_INT_EN		0x00000080ul	/* GT3 */
-#define CRTC_SNAPSHOT_INT		0x00000100ul	/* GT3 */
-#define CRTC_I2C_INT_EN			0x00000200ul	/* GT3 */
-#define CRTC_I2C_INT			0x00000400ul	/* GT3 */
-#define CRTC_CAPBUF0_INT_EN		0x00010000ul	/* VT/GT */
-#define CRTC_CAPBUF0_INT		0x00020000ul	/* VT/GT */
-#define CRTC_CAPBUF1_INT_EN		0x00040000ul	/* VT/GT */
-#define CRTC_CAPBUF1_INT		0x00080000ul	/* VT/GT */
-#define CRTC_OVERLAY_EOF_INT_EN		0x00100000ul	/* VT/GT */
-#define CRTC_OVERLAY_EOF_INT		0x00200000ul	/* VT/GT */
-#define CRTC_ONESHOT_CAP_INT_EN		0x00400000ul	/* VT/GT */
-#define CRTC_ONESHOT_CAP_INT		0x00800000ul	/* VT/GT */
-#define CRTC_BUSMASTER_EOL_INT_EN	0x01000000ul	/* VTB/GTB/LT */
-#define CRTC_BUSMASTER_EOL_INT		0x02000000ul	/* VTB/GTB/LT */
-#define CRTC_GP_INT_EN			0x04000000ul	/* VTB/GTB/LT */
-#define CRTC_GP_INT			0x08000000ul	/* VTB/GTB/LT */
-/*	?				0xf0000000ul */
-#define CRTC_VBLANK_BIT2		0x80000000ul	/* GT3 */
-#define CRTC_INT_ENS	/* *** UPDATE ME *** */		\
-		(					\
-			CRTC_VBLANK_INT_EN |		\
-			CRTC_VLINE_INT_EN |		\
-			CRTC_SNAPSHOT_INT_EN |		\
-			CRTC_I2C_INT_EN |		\
-			CRTC_CAPBUF0_INT_EN |		\
-			CRTC_CAPBUF1_INT_EN |		\
-			CRTC_OVERLAY_EOF_INT_EN |	\
-			CRTC_ONESHOT_CAP_INT_EN |	\
-			CRTC_BUSMASTER_EOL_INT_EN |	\
-			CRTC_GP_INT_EN |		\
-			0				\
-		)
-#define CRTC_INT_ACKS	/* *** UPDATE ME *** */		\
-		(					\
-			CRTC_VBLANK_INT |		\
-			CRTC_VLINE_INT |		\
-			CRTC_SNAPSHOT_INT |		\
-			CRTC_I2C_INT |			\
-			CRTC_CAPBUF0_INT |		\
-			CRTC_CAPBUF1_INT |		\
-			CRTC_OVERLAY_EOF_INT |		\
-			CRTC_ONESHOT_CAP_INT |		\
-			CRTC_BUSMASTER_EOL_INT |	\
-			CRTC_GP_INT |			\
-			0				\
-		)
-#define CRTC_DBL_SCAN_EN		0x00000001ul
-#define CRTC_INTERLACE_EN		0x00000002ul
-#define CRTC_HSYNC_DIS			0x00000004ul
-#define CRTC_VSYNC_DIS			0x00000008ul
-#define CRTC_CSYNC_EN			0x00000010ul
-#define CRTC_PIX_BY_2_EN		0x00000020ul
-#define CRTC_DISPLAY_DIS		0x00000040ul
-#define CRTC_VGA_XOVERSCAN		0x00000080ul
-#define CRTC_PIX_WIDTH			0x00000700ul
-#define CRTC_PIX_WIDTH_1BPP			0x00000000ul
-#define CRTC_PIX_WIDTH_4BPP			0x00000100ul
-#define CRTC_PIX_WIDTH_8BPP			0x00000200ul
-#define CRTC_PIX_WIDTH_15BPP			0x00000300ul
-#define CRTC_PIX_WIDTH_16BPP			0x00000400ul
-#define CRTC_PIX_WIDTH_24BPP			0x00000500ul
-#define CRTC_PIX_WIDTH_32BPP			0x00000600ul
-/*	?					0x00000700ul */
-#define CRTC_BYTE_PIX_ORDER		0x00000800ul
-#define CRTC_FIFO_OVERFILL		0x0000c000ul	/* VT/GT */
-#define CRTC_FIFO_LWM			0x000f0000ul
-#define CRTC_VGA_128KAP_PAGING		0x00100000ul	/* VT/GT */
-#define CRTC_DISPREQ_ONLY		0x00200000ul	/* VT/GT */
-#define CRTC_VFC_SYNC_TRISTATE		0x00200000ul	/* VTB/GTB/LT */
-#define CRTC_LOCK_REGS			0x00400000ul	/* VT/GT */
-#define CRTC_SYNC_TRISTATE		0x00800000ul	/* VT/GT */
-#define CRTC_EXT_DISP_EN		0x01000000ul
-#define CRTC_EN				0x02000000ul
-#define CRTC_DISP_REQ_EN		0x04000000ul
-#define CRTC_VGA_LINEAR			0x08000000ul
-#define CRTC_VSYNC_FALL_EDGE		0x10000000ul
-#define CRTC_VGA_TEXT_132		0x20000000ul
-#define CRTC_CNT_EN			0x40000000ul
-#define CRTC_CUR_B_TEST			0x80000000ul
-#define DSP_OFF				0x000007fful
-#define CRTC_PITCH			0xffc00000ul
-#define CTL_MEM_SIZE			0x00000007ul	
-#define CTL_MEM_SIZEB			0x0000000ful	/* VTB/GTB/LT */
-#define PLL_WR_EN			0x00000200ul	/* For internal PLL */
-#define PLL_ADDR			0x00007c00ul	/* For internal PLL */
-#define BUS_HOST_ERR_INT_EN		0x00400000ul
-#define BUS_HOST_ERR_INT		0x00800000ul
-#define BUS_APER_REG_DIS		0x00000010ul	/* VTB/GTB/LT */
-#define CLOCK_SELECT			0x0000000ful
-#define CLOCK_DIVIDER			0x00000030ul
-#define CLOCK_STROBE			0x00000040ul
-
-#define GEN_CUR_EN			0x00000080ul
-
-#define BUS_WS				0x0000000ful
-#define BUS_DBL_RESYNC			0x00000001ul	/* VTB/GTB/LT */
-#define BUS_MSTR_RESET			0x00000002ul	/* VTB/GTB/LT */
-#define BUS_FLUSH_BUF			0x00000004ul	/* VTB/GTB/LT */
-#define BUS_STOP_REQ_DIS		0x00000008ul	/* VTB/GTB/LT */
-#define BUS_ROM_WS			0x000000f0ul
-#define BUS_APER_REG_DIS		0x00000010ul	/* VTB/GTB/LT */
-#define BUS_EXTRA_PIPE_DIS		0x00000020ul	/* VTB/GTB/LT */
-#define BUS_MASTER_DIS			0x00000040ul	/* VTB/GTB/LT */
-#define BUS_ROM_WRT_EN			0x00000080ul	/* GT3 */
-#define BUS_ROM_PAGE			0x00000f00ul
-#define BUS_ROM_DIS			0x00001000ul
-#define BUS_IO_16_EN			0x00002000ul	/* GX */
-#define BUS_PCI_READ_RETRY_EN		0x00002000ul	/* VTB/GTB/LT */
-#define BUS_DAC_SNOOP_EN		0x00004000ul
-#define BUS_PCI_RETRY_EN		0x00008000ul	/* VT/GT */
-#define BUS_PCI_WRT_RETRY_EN		0x00008000ul	/* VTB/GTB/LT */
-#define BUS_FIFO_WS			0x000f0000ul
-#define BUS_RETRY_WS			0x000f0000ul	/* VTB/GTB/LT */
-#define BUS_FIFO_ERR_INT_EN		0x00100000ul
-#define BUS_MSTR_RD_MULT		0x00100000ul	/* VTB/GTB/LT */
-#define BUS_FIFO_ERR_INT		0x00200000ul
-#define BUS_MSTR_RD_LINE		0x00200000ul	/* VTB/GTB/LT */
-#define BUS_HOST_ERR_INT_EN		0x00400000ul
-#define BUS_SUSPEND			0x00400000ul	/* GT3 */
-#define BUS_HOST_ERR_INT		0x00800000ul
-#define BUS_LAT16X			0x00800000ul	/* GT3 */
-#define BUS_PCI_DAC_WS			0x07000000ul
-#define BUS_RD_DISCARD_EN		0x01000000ul	/* VTB/GTB/LT */
-#define BUS_RD_ABORT_EN			0x02000000ul	/* VTB/GTB/LT */
-#define BUS_MSTR_WS			0x04000000ul	/* VTB/GTB/LT */
-#define BUS_PCI_DAC_DLY			0x08000000ul
-#define BUS_EXT_REG_EN			0x08000000ul	/* VT/GT */
-#define BUS_PCI_MEMW_WS			0x10000000ul
-#define BUS_MSTR_DISCONNECT_EN		0x10000000ul	/* VTB/GTB/LT */
-#define BUS_PCI_BURST_DEC		0x20000000ul	/* GX/CX */
-#define BUS_BURST			0x20000000ul	/* 264xT */
-#define BUS_WRT_BURST			0x20000000ul	/* VTB/GTB/LT */
-#define BUS_RDY_READ_DLY		0xc0000000ul
-#define BUS_READ_BURST			0x40000000ul	/* VTB/GTB/LT */
-#define BUS_RDY_READ_DLY_B		0x80000000ul	/* VTB/GTB/LT */
-
-#define CFG_MEM_VGA_AP_EN		0x00000004ul
-#define DSP_XCLKS_PER_QW		0x00003ffful
-#define DSP_FLUSH_WB			0x00008000ul
-#define DSP_LOOP_LATENCY		0x000f0000ul
-#define DSP_PRECISION			0x00700000ul
-#define DSP_OFF				0x000007fful
-#define DSP_ON				0x07ff0000ul
-
-#define PLL_VCLK_CNTL		0x05u
-#define PLL_VCLK_POST_DIV	0x06u
-#define PLL_VCLK0_FB_DIV	0x07u
-#define PLL_VCLK0_XDIV		0x10u
-#define PLL_XCLK_CNTL		0x0bu	/* VT/GT */
-#define PLL_VCLK_RESET		0x04u
-#define PLL_XCLK_SRC_SEL	0x07u	/* VTB/GTB/LT */
-#define PLL_MCLK_FB_DIV		0x04u
-#define PLL_MFB_TIMES_4_2B	0x08u
-#define PLL_REF_DIV		0x02u
-
-#define CTL_MEM_TRAS			0x00070000ul	/* VTB/GTB/LT */
-#define CTL_MEM_TRCD			0x00000c00ul	/* VTB/GTB/LT */
-#define CTL_MEM_TCRD			0x00001000ul	/* VTB/GTB/LT */
-#define CTL_MEM_TRP			0x00000300ul	/* VTB/GTB/LT */
-
-#define CTL_MEM_BNDRY			0x00030000ul
-#define CTL_MEM_BNDRY_EN		0x00040000ul
-
-#define ATI_CHIP_88800GXC 16    /* Mach64 */
-#define ATI_CHIP_88800GXD 17    /* Mach64 */
-#define ATI_CHIP_88800GXE 18    /* Mach64 */
-#define ATI_CHIP_88800GXF 19    /* Mach64 */
-#define ATI_CHIP_88800GX  20    /* Mach64 */
-#define ATI_CHIP_88800CX  21    /* Mach64 */
-#define ATI_CHIP_264CT    22    /* Mach64 */
-#define ATI_CHIP_264ET    23    /* Mach64 */
-#define ATI_CHIP_264VT    24    /* Mach64 */
-#define ATI_CHIP_264GT    25    /* Mach64 */
-#define ATI_CHIP_264VTB   26    /* Mach64 */
-#define ATI_CHIP_264GTB   27    /* Mach64 */
-#define ATI_CHIP_264VT3   28    /* Mach64 */
-#define ATI_CHIP_264GTDVD 29    /* Mach64 */
-#define ATI_CHIP_264LT    30    /* Mach64 */
-#define ATI_CHIP_264VT4   31    /* Mach64 */
-#define ATI_CHIP_264GT2C  32    /* Mach64 */
-#define ATI_CHIP_264GTPRO 33    /* Mach64 */
-#define ATI_CHIP_264LTPRO 34    /* Mach64 */
-#define ATI_CHIP_Mach64   35    /* Mach64 */
-
-#define MEM_264_NONE            0
-#define MEM_264_DRAM            1
-#define MEM_264_EDO             2
-#define MEM_264_PSEUDO_EDO      3
-#define MEM_264_SDRAM           4
-#define MEM_264_SGRAM           5
-#define MEM_264_TYPE_6          6
-#define MEM_264_TYPE_7          7
-
-#define ATIGetMach64PLLReg(_Index)                              \
-        (                                                       \
-                ATIAccessMach64PLLReg(_Index, 0),           \
-                inb(ATIIOPortCLOCK_CNTL + 2)                    \
-        )
-#define ATIPutMach64PLLReg(_Index, _Value)                      \
-        (                                                       \
-                ATIAccessMach64PLLReg(_Index, 1),            \
-                outb(ATIIOPortCLOCK_CNTL + 2, _Value)           \
-        )
+#define RAGE_TOTAL_REGS (VGA_TOTAL_REGS + 1000)
 
 int           ATIIOPortCRTC_H_TOTAL_DISP, ATIIOPortCRTC_H_SYNC_STRT_WID,
               ATIIOPortCRTC_V_TOTAL_DISP, ATIIOPortCRTC_V_SYNC_STRT_WID,
@@ -868,7 +547,7 @@ static void rage_setregs(const unsigned char regs[], int mode)
     };
 
     /* make sure the dac is in 8 bit or 6 bit mode, as needed */
-    outl(ATIIOPortDAC_CNTL, restore->dac_cntl);
+    outl(ATIIOPortDAC_CNTL, restore->dac_cntl); 
  
     outb(ATIIOPortDAC_MASK, 0xFFU);
     outb(ATIIOPortDAC_WRITE, 0x00U);
@@ -900,6 +579,8 @@ static int rage_modeavailable(int mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
+    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
+
     if ((mode < G640x480x256 )
 	|| mode == G720x348x2)
 	return __svgalib_vga_driverspecs.modeavailable(mode);
@@ -907,7 +588,6 @@ static int rage_modeavailable(int mode)
     info = &__svgalib_infotable[mode];
     if (rage_memory * 1024 < info->ydim * info->xbytes)
 	return 0;
-
     modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
     modetiming = malloc(sizeof(ModeTiming));
     if (__svgalib_getmodetiming(modetiming, modeinfo, cardspecs)) {
@@ -1108,6 +788,8 @@ static void rage_initializemode(unsigned char *moderegs,
        break;       
     };
 
+
+
     rage_bpp = modeinfo->bytesPerPixel; 
 
 if((ATIChip>=ATI_CHIP_264VTB)&&(ATIIODecoding==BLOCK_IO))
@@ -1318,13 +1000,6 @@ static int rage_test(void)
     int i, found;
     unsigned long buf[64];
     
-    if (getenv("IOPERM") == NULL) {
-      if (iopl(3) < 0) {
-        printf("svgalib: rage: cannot get I/O permissions\n");
-        exit(1);
-      }
-    }
-
    found=__svgalib_pci_find_vendor_vga(0x1002,buf,0);
    if(found)return 0;
    if (!found){
@@ -1436,7 +1111,6 @@ static int rage_init(int force, int par1, int par2)
 {
    unsigned long buf[64];
    int found=0;
-   int _ioperm=0;
    int i,j;
    unsigned char *BIOS;
 
@@ -1451,14 +1125,6 @@ static int rage_init(int force, int par1, int par2)
 
    };
 
-   if (getenv("IOPERM") == NULL) {
-     _ioperm=1;
-     if (iopl(3) < 0) {
-       printf("svgalib: rage: cannot get I/O permissions\n");
-       exit(1);
-     }
-   }
-
    found=__svgalib_pci_find_vendor_vga(0x1002,buf,0);
    rage_linear_base=0;
    if (!found){
@@ -1592,8 +1258,7 @@ static int rage_init(int force, int par1, int par2)
          cardspecs->maxPixelClock32bpp = 40000;
    };
    cardspecs->flags = CLOCK_PROGRAMMABLE;
-   cardspecs->maxHorizontalCrtc = 4088;
-   cardspecs->nClocks = 0;
+   cardspecs->maxHorizontalCrtc = 2040;
    cardspecs->maxPixelClock4bpp = 0;
    cardspecs->mapClock = rage_map_clock;
    cardspecs->mapHorizontalCrtc = rage_map_horizontal_crtc;
@@ -1604,52 +1269,13 @@ static int rage_init(int force, int par1, int par2)
    __svgalib_linear_mem_base=rage_linear_base;
    __svgalib_linear_mem_size=rage_memory*0x400;
 
-#define BIOSWord(x) ((int)BIOS[x]+256*(int)BIOS[x+1])
-
    BIOS=mmap(0,32*1024,PROT_READ|PROT_WRITE,MAP_SHARED,__svgalib_mem_fd,0xc0000);
 
-   i=BIOSWord(0x48);
-   j=BIOSWord(i+0x10);
+   i=(int)BIOS[0x48]+256*(int)BIOS[0x49];
+   j=(int)BIOS[i+0x10]+256*(int)BIOS[i+0x11];
 
    ATIClockToProgram=BIOS[j+6];
 
-   if(ATIChip>=ATI_CHIP_264CT){
-      M=ATIGetMach64PLLReg(PLL_REF_DIV);
-      minN=2;
-      maxN=255;
-      switch (BIOSWord(j+0x08)/10) {
-         case 143:
-            fref=14318; 
-            break;
-         case 286:
-            fref=28636;
-            break;
-         default:
-            fref=BIOSWord(j+0x08)*10;
-            break;
-      }
-      Nadj=0;
-      fref*=2; /* X says double for all chips */
-#if 0
-      if(__svgalib_ragedoubleclock)fref/=2; /* just in case */
-#endif
-      if (__svgalib_driver_report) {
-          printf("Rage: BIOS reports base frequency=%.3fMHz  Denominator=%3i\n",fref/1000,M);
-      }
-   } else {
-      rage_dac=(j>>9)&7;
-      if(rage_dac==5) {
-         rage_clock=1;
-         M=46;
-         minN=257;
-         maxN=512;
-         Nadj=257;
-         for(i=0;i<4;i++)postdiv[i]=1<<i;
-         for(i=4;i<8;i++)postdiv[i]=0;
-         fref=14318;
-      };
-   };
-   
    munmap(BIOS,32768);
 
    return 0;
