@@ -35,9 +35,10 @@ chipset companies still do this).
 #include "interface.h"
 #include "accel.h"
 #include "vgapci.h"
+#include "vgarelvgaio.h"
 
 #define SISREG_SAVE(i) (VGA_TOTAL_REGS+i)
-#define SIS_TOTAL_REGS (VGA_TOTAL_REGS + 0x48 - 5)
+#define SIS_TOTAL_REGS (VGA_TOTAL_REGS + 64)
 
 #define XR(i) (VGA_TOTAL_REGS+i-5)
 
@@ -50,9 +51,6 @@ chipset companies still do this).
 #define PCI_CHIP_SIS5597        0x0200
 #define PCI_CHIP_SIS6326        0x6326
 #define PCI_CHIP_SIS530        0x6306
-#define PCI_CHIP_SIS300         0x0300
-#define PCI_CHIP_SIS540         0x5300
-#define PCI_CHIP_SIS630         0x6300
 
 #define ClockReg	XR(0x07) 
 #define DualBanks	XR(0x0B)
@@ -66,8 +64,8 @@ chipset companies still do this).
 enum { SIS_86C201=0, SIS_86C202, SIS_86C205, SIS_86C215, SIS_86C225,
        SIS_5597, SIS_5598, SIS_6326, SIS_530 };
 
-#define write_xr(num,val) {outb(SEQ_I, num);outb(SEQ_D, val);}
-#define read_xr(num,var) {outb(SEQ_I, num);var=inb(SEQ_D);} 
+#define write_xr(num,val) {__svgalib_outseq(num,val);}
+#define read_xr(num,var) {var=__svgalib_inseq(num);} 
 
 static int sis_init(int, int, int);
 static void sis_unlock(void);
@@ -88,14 +86,8 @@ static CardSpecs *cardspecs;
 
 static void sis_setpage(int page)
 {
-        char tmp;
-
-        read_xr(0x0b,tmp);
-        if (tmp&8) {
-            outb(0x3cb,page);
-            outb(0x3cd,page);
-        } else
-            outb(0x3cd,page|(page<<4));
+        outb(0x3cb+io_reloc,page);
+        outb(0x3cd+io_reloc,page);
 }
 
 static int __svgalib_sis_inlinearmode(void)
@@ -129,38 +121,18 @@ static int sis_saveregs(unsigned char regs[])
 { 
     int i;
 
-    sis_unlock();
+    sis_unlock();		
+
+    for(i=5; i<0x38;i++)read_xr(i,regs[VGA_TOTAL_REGS+i-5]);
     
-    regs[XR(0x40)] = inb(0x3cb);
-    regs[XR(0x41)] = inb(0x3cd);	
-
     if(sis_chiptype==SIS_5597)
-       for(i=0x38; i<0x3a;i++)read_xr(i,regs[XR(i)]);
-
+       for(i=0x38; i<0x3a;i++)read_xr(i,regs[VGA_TOTAL_REGS+i-5]);
+ 
     if(sis_chiptype==SIS_6326)
-       for(i=0x38; i<0x3d;i++)read_xr(i,regs[XR(i)]);
+       for(i=0x38; i<0x3d;i++)read_xr(i,regs[VGA_TOTAL_REGS+i-5]);
 
     if(sis_chiptype==SIS_530)
-       for(i=0x38; i<0x40 ; i++)read_xr(i,regs[XR(i)]);
-
-    /* get all the clocks */
-    if (sis_chiptype==SIS_5597 || sis_chiptype==SIS_6326 || sis_chiptype==SIS_530) {
-       i = regs[XR(0x38)] & 0xfc;
-       write_xr(0x38,i|1);
-       read_xr(0x13,regs[XR(0x42)]);
-       read_xr(0x2a,regs[XR(0x43)]);
-       read_xr(0x2b,regs[XR(0x44)]);
-       write_xr(0x38,i|2);
-       read_xr(0x13,regs[XR(0x45)]);
-       read_xr(0x2a,regs[XR(0x46)]);
-       read_xr(0x2b,regs[XR(0x47)]);
-       write_xr(0x38,i); /* Make sure that the loop gets the internal VCLK registers */
-    }
-
-    for(i=5; i<0x38;i++)read_xr(i,regs[XR(i)]);
-    
-    if (sis_chiptype==SIS_5597 || sis_chiptype==SIS_6326 || sis_chiptype==SIS_530)
-       write_xr(0x38,regs[XR(0x38)]);
+       for(i=0x38; i<0x40 ; i++)read_xr(i,regs[VGA_TOTAL_REGS+i-5]);
 
     return SIS_TOTAL_REGS - VGA_TOTAL_REGS;
 }
@@ -173,33 +145,19 @@ static void sis_setregs(const unsigned char regs[], int mode)
 
     sis_unlock();
     
-    outb(0x3cb,regs[XR(0x40)]);
-    outb(0x3cd,regs[XR(0x41)]);
+    write_xr(0x5,0x86);
 
-    /* set all the clocks */
-    if (sis_chiptype==SIS_5597 || sis_chiptype==SIS_6326 || sis_chiptype==SIS_530) {
-       i = regs[XR(0x38)] & 0xfc;
-       write_xr(0x38,i|1);
-       write_xr(0x13,regs[XR(0x42)]);
-       write_xr(0x2a,regs[XR(0x43)]);
-       write_xr(0x2b,regs[XR(0x44)]);
-       write_xr(0x38,i|2);
-       write_xr(0x13,regs[XR(0x45)]);
-       write_xr(0x2a,regs[XR(0x46)]);
-       write_xr(0x2b,regs[XR(0x47)]);
-       write_xr(0x38,i); /* Make sure that the loop puts the internal VCLK registers */
-    }
-
-    for(i=6; i<0x38;i++)write_xr(i,regs[XR(i)]);
+    for(i=6; i<0x38;i++)write_xr(i,regs[VGA_TOTAL_REGS+i-5]);
     
     if(sis_chiptype==SIS_5597)
-       for(i=0x38; i<0x3a;i++)write_xr(i,regs[XR(i)]);
+       for(i=0x38; i<0x3a;i++)write_xr(i,regs[VGA_TOTAL_REGS+i-5]);
  
     if(sis_chiptype==SIS_6326)
-       for(i=0x38; i<0x3d;i++)write_xr(i,regs[XR(i)]);
+       for(i=0x38; i<0x3d;i++)write_xr(i,regs[VGA_TOTAL_REGS+i-5]);
 
     if(sis_chiptype==SIS_530)
-       for(i=0x38; i<0x40;i++)write_xr(i,regs[XR(i)]);
+       for(i=0x38; i<0x40;i++)write_xr(i,regs[VGA_TOTAL_REGS+i-5]);
+
 }
 
 
@@ -211,6 +169,8 @@ static int sis_modeavailable(int mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
+    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
+
     if ((mode < G640x480x256 )
 	|| mode == G720x348x2)
 	return __svgalib_vga_driverspecs.modeavailable(mode);
@@ -339,6 +299,7 @@ static void sis_initializemode(unsigned char *moderegs,
         }
     }
     
+
     moderegs[59]=0x6f;
 
     return ;
@@ -350,30 +311,14 @@ static int sis_setmode(int mode, int prv_mode)
     unsigned char *moderegs;
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
-    int tmp, var;
+    int tmp;
 
     if ((mode < G640x480x256)||(mode==G720x348x2)) {
         if(sis_chiptype >= SIS_5597) { 
-		/* Work Around for vesafb case */
+		/* Work Arrout for vesafb case */
                 read_xr(0x06,tmp);
                 tmp&=~(0x1E); /* Depth registers */
                 write_xr(0x06,tmp);
-		/* program the 25 and 28 Mhz clocks */
-		if (sis_chiptype==SIS_5597 || sis_chiptype==SIS_6326 || sis_chiptype==SIS_530) {
-                   read_xr(0x38,tmp);
-                   tmp &= 0xfc;
-                   write_xr(0x38,tmp|1);
-	           read_xr(0x13,var);
-                   write_xr(0x13,var|0x40);
-                   write_xr(0x2a,0x1b);
-                   write_xr(0x2b,0xe1);
-                   write_xr(0x38,tmp|2);
-                   read_xr(0x13,var);
-                   write_xr(0x13,var|0x40);
-                   write_xr(0x2a,0x4e);
-                   write_xr(0x2b,0xe4);
-                   write_xr(0x38,tmp);
-		}
         }
  
 	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
@@ -412,28 +357,14 @@ static int sis_setmode(int mode, int prv_mode)
 
 static void sis_unlock(void)
 {
-    int vgaIOBase, temp;
-
-    vgaIOBase = (inb(0x3CC) & 0x01) ? 0x3D0 : 0x3B0;
-    outb(vgaIOBase + 4, 0x11);
-    temp = inb(vgaIOBase + 5);
-    outb(vgaIOBase + 5, temp & 0x7F);
-
-    outw(SEQ_I,0x8605);
-    
+    __svgalib_outcrtc(0x11,__svgalib_incrtc(0x11)&0x7f);
+    __svgalib_outseq(0x05,0x86);
 }
 
 static void sis_lock(void)
 {
-    int vgaIOBase, temp;
-
-    vgaIOBase = (inb(0x3CC) & 0x01) ? 0x3D0 : 0x3B0;
-    outb(vgaIOBase + 4, 0x11);
-    temp = inb(vgaIOBase + 5);
-    outb(vgaIOBase + 5, temp & 0x7F);   
-
-    outw(SEQ_I,0x0005);
-
+    __svgalib_outcrtc(0x11,__svgalib_incrtc(0x11)&0x7f);
+    __svgalib_outseq(0x05,0x0);
 }
 
 #define VENDOR_ID 0x1039
@@ -442,21 +373,11 @@ static void sis_lock(void)
 
 static int sis_test(void)
 {
-    int _ioperm=0, found;
+    int found;
     long buf[64];
     
-    if (getenv("IOPERM") == NULL) {
-      _ioperm=1;
-      if (iopl(3) < 0) {
-        printf("svgalib: cannot get I/O permissions\n");
-        exit(1);
-      }
-    }
-    
     found=__svgalib_pci_find_vendor_vga(VENDOR_ID,buf,0);
     
-    if (_ioperm) iopl(0);
-    
     if(!found&&(
        (((buf[0]>>16)&0xffff)==PCI_CHIP_SG86C201)||
        (((buf[0]>>16)&0xffff)==PCI_CHIP_SG86C202)||
@@ -466,9 +387,6 @@ static int sis_test(void)
        (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS5597)||
        (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS5598)||
        (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS530)||
-       (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS540)||
-       (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS630)||
-       (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS300)||
        (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS6326)
        )){
        sis_init(0,0,0);
@@ -485,8 +403,8 @@ static void sis_setdisplaystart(int address)
 { 
   int temp;
   address=address >> 2;
-  outw(CRT_IC, (address & 0x00FF00) | 0x0C);
-  outw(CRT_IC, ((address & 0x00FF) << 8) | 0x0D);
+  __svgalib_outcrtc(0x0c,(address & 0x00FF00)>>8);
+  __svgalib_outcrtc(0x0d,address & 0x0000ff);
 
   read_xr(0x27,temp);
   temp &= 0xf0;
@@ -503,7 +421,7 @@ static void sis_setlogicalwidth(int width)
     int offset = width >> 3;
     int temp;
     
-    __svgalib_outCR(0x13,offset&0xff);
+    __svgalib_outcrtc(0x13,offset & 0xff);
     read_xr(0x0a,temp);
     temp &= 0xf;
     temp |= (offset&0xf00)>>4;
@@ -591,7 +509,6 @@ static int sis_init(int force, int par1, int par2)
 {
     unsigned long buf[64];
     int found=0;
-    int _ioperm=0;
 
     sis_unlock();
     if (force) {
@@ -601,15 +518,7 @@ static int sis_init(int force, int par1, int par2)
 
     };
 
-    if (getenv("IOPERM") == NULL) {
-      _ioperm=1;
-      if (iopl(3) < 0) {
-        printf("svgalib: cannot get I/O permissions\n");
-        exit(1);
-      }
-    }
     found=__svgalib_pci_find_vendor_vga(VENDOR_ID,buf,0);
-    if (_ioperm) iopl(0);
     sis_linear_base=0;
     sis_chiptype=0;
     if (!found){
@@ -621,15 +530,17 @@ static int sis_init(int force, int par1, int par2)
             case PCI_CHIP_SG86C215: sis_chiptype=SIS_86C215; break;
             case PCI_CHIP_SG86C225: sis_chiptype=SIS_86C225; break;
             case PCI_CHIP_SIS5597: sis_chiptype=SIS_5597; break;
-            case PCI_CHIP_SIS300:
-            case PCI_CHIP_SIS540:
-            case PCI_CHIP_SIS630:
             case PCI_CHIP_SIS530: sis_chiptype=SIS_530; break;
 /*            case PCI_CHIP_SIS5598: sis_chiptype=SIS_5598; break;*/
             case PCI_CHIP_SIS6326: sis_chiptype=SIS_6326; break;
        };
     };
-
+    
+    if(sis_chiptype>=SIS_5597) {
+       io_reloc=(buf[6]&0xffffff80)-0x380; /* should work at least for 6326 and 530 */
+       rel_io_mapio();
+    }
+    
     if(sis_memory==0){
         if ((sis_chiptype == SIS_6326)||(sis_chiptype==SIS_530)){
             int temp;
@@ -663,6 +574,24 @@ static int sis_init(int force, int par1, int par2)
 	printf("Using SIS driver, %iKB. Chiptype=%i\n",sis_memory,sis_chiptype);
     };
 
+    /* program the 25 and 28 Mhz clocks */
+    if ((sis_chiptype==SIS_5597)||(sis_chiptype==SIS_6326)||(sis_chiptype==SIS_530)) {
+       int temp,var;
+       read_xr(0x38,temp);
+       temp &= 0xfc;
+       write_xr(0x38,temp|1);
+       read_xr(0x13,var);
+       write_xr(0x13,var|0x40);
+       write_xr(0x2a,0x1b);
+       write_xr(0x2b,0xe1);
+       write_xr(0x38,temp|2);
+       read_xr(0x13,var);
+       write_xr(0x13,var|0x40);
+       write_xr(0x2a,0x4e);
+       write_xr(0x2b,0xe4);
+       write_xr(0x38,temp);
+    }
+
     cardspecs = malloc(sizeof(CardSpecs));
     cardspecs->videoMemory = sis_memory;
     cardspecs->maxPixelClock4bpp = 75000;	
