@@ -1,7 +1,6 @@
 /*
 */
 
-
 #include <stdlib.h>
 #include <stdio.h>		/* for printf */
 #include <string.h>		/* for memset */
@@ -11,7 +10,6 @@
 #include "libvga.h"
 #include "driver.h"
 
-
 /* New style driver interface. */
 #include "timing.h"
 #include "vgaregs.h"
@@ -34,7 +32,7 @@ static int vesa_is_linear, vesa_logical_width, vesa_bpp, vesa_granularity;
 static int vesa_regs_size, vesa_linear_base, vesa_last_mode_set;
 static struct LRMI_regs vesa_r;
 static int vesa_read_write, vesa_read_window, vesa_write_window;
-static void * LRMI_mem1;
+static void * LRMI_mem1, * LRMI_mem2;
 
 static CardSpecs *cardspecs;
 static struct
@@ -45,8 +43,6 @@ static struct
 
 static int SVGALIB_VESA[__GLASTMODE+1];
 
-static int lrmi_inited=0;
-
 static void vesa_setpage(int page)
 {
 vesa_r.eax=0x4f05;
@@ -73,9 +69,6 @@ return vesa_is_linear;
 
 static void vesa_getmodeinfo(int mode, vga_modeinfo *modeinfo)
 {
-    if ((mode < G640x480x256 )
-	|| mode == G720x348x2)
-	return __svgalib_vga_driverspecs.getmodeinfo(mode, modeinfo);
 
     if(modeinfo->colors==16)return;
 
@@ -94,19 +87,6 @@ static void vesa_getmodeinfo(int mode, vga_modeinfo *modeinfo)
 	    modeinfo->flags |= IS_LINEAR;
       }
       
-    /* to get the logical scanline width */
-    memset(&vesa_r, 0, sizeof(vesa_r));
-
-    vesa_r.eax = 0x4f01;
-    vesa_r.ecx = SVGALIB_VESA[mode];
-    vesa_r.es = (unsigned int)vesa_data.mode >> 4;
-    vesa_r.edi = (unsigned int)vesa_data.mode & 0xf;
-
-    if (!__svgalib_LRMI_int(0x10, &vesa_r)) {
-       fprintf(stderr, "Can't get mode info (vm86 failure)\n");
-       return;
-    }
-    modeinfo->linewidth = vesa_data.mode->bytes_per_scanline;
 }
 
 /* Read and save chipset-specific registers */
@@ -150,6 +130,8 @@ static int vesa_modeavailable(int mode)
     ModeInfo *modeinfo;
 
 
+    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
+
     if ((mode < G640x480x256 )
 	|| mode == G720x348x2)
 	return __svgalib_vga_driverspecs.modeavailable(mode);
@@ -170,6 +152,7 @@ static int vesa_modeavailable(int mode)
 return SVGALIB_VESA[mode];
 }
 
+
 static int vesa_setmode(int mode, int prv_mode)
 {
     vesa_bpp=1;
@@ -189,6 +172,8 @@ static int vesa_setmode(int mode, int prv_mode)
     vesa_last_mode_set=vesa_r.ebx;
     __svgalib_LRMI_int(0x10,&vesa_r);
 
+    vesa_data.info = LRMI_mem2 ;
+    vesa_data.mode = (struct vbe_mode_info_block *)(vesa_data.info + 1);
     vesa_r.eax = 0x4f01;
     vesa_r.ecx=SVGALIB_VESA[mode];
     vesa_r.es = (unsigned int)vesa_data.mode >> 4;
@@ -233,23 +218,16 @@ static void vesa_lock(void)
 static int vesa_test(void)
 {
 
-    if ((getenv("IOPERM") == NULL) && ((iopl(3) < 0)||(ioperm(0,0x400,1)<0))) {
-	printf("svgalib(vesa): Cannot get I/O permissions.\n");
-	exit(-1);
-    }
-    
-    lrmi_inited=1;
-    __svgalib_LRMI_init();
-    vesa_data.info = __svgalib_LRMI_alloc_real(sizeof(struct vbe_info_block)
-	 + sizeof(struct vbe_mode_info_block));
-    vesa_data.mode = (struct vbe_mode_info_block *)(vesa_data.info + 1);
-    vesa_r.eax = 0x4f00;
-    vesa_r.es = (unsigned int)vesa_data.info >> 4;
-    vesa_r.edi = 0;
+        __svgalib_LRMI_init();
+	vesa_data.info = LRMI_mem2;
+	vesa_data.mode = (struct vbe_mode_info_block *)(vesa_data.info + 1);
+	vesa_r.eax = 0x4f00;
+	vesa_r.es = (unsigned int)vesa_data.info >> 4;
+	vesa_r.edi = 0;
 
-    __svgalib_LRMI_int(0x10, &vesa_r);
-    if (vesa_r.eax!=0x4f) return 0;
-    return !vesa_init(0,0,0);
+        __svgalib_LRMI_int(0x10, &vesa_r);
+       if (vesa_r.eax!=0x4f) return 0;
+       return !vesa_init(0,0,0);
 }
 
 
@@ -365,8 +343,9 @@ static int vesa_init(int force, int par1, int par2)
     int i;
     unsigned char *m;
     int address;
-    
-    m=mmap(0,1024,PROT_READ,MAP_SHARED,__svgalib_mem_fd,0);
+
+#if 0
+    m=mmap(0,0x502,PROT_READ,MAP_SHARED,__svgalib_mem_fd,0);
     
     address=((long)*(unsigned short *)(m+64))+(((long)*(unsigned short *)(m+66))<<4);
     if((address<0xa0000)||(address>0xfffff)) {
@@ -374,16 +353,11 @@ static int vesa_init(int force, int par1, int par2)
         fprintf(stderr, "Try running vga_reset.\n");
        	return 1;
     };
-
 printf("%x\n",address    );
-
+#endif
     __svgalib_textprog|=1;
 
     /* Get I/O priviledge */
-    if ((getenv("IOPERM") == NULL) && ((iopl(3) < 0)||(ioperm(0,0x400,1)<0))) {
-	printf("svgalib(vesa): Cannot get I/O permissions.\n");
-	exit(-1);
-    }
 
     if (force) {
 	vesa_memory = par1;
@@ -392,15 +366,12 @@ printf("%x\n",address    );
         vesa_memory=4096; 
     };
 
-    if(!lrmi_inited) {
-        __svgalib_LRMI_init();
-        vesa_data.info = __svgalib_LRMI_alloc_real(sizeof(struct vbe_info_block)
-	                 + sizeof(struct vbe_mode_info_block));
-        vesa_data.mode = (struct vbe_mode_info_block *)(vesa_data.info + 1);
-        lrmi_inited=1;
-    }
+    __svgalib_LRMI_init();
     for(i=0;i<__GLASTMODE;i++)SVGALIB_VESA[i]=((i<G640x480x256)||(i==G720x348x2));
 
+    vesa_data.info = __svgalib_LRMI_alloc_real(sizeof(struct vbe_info_block)
+	 + sizeof(struct vbe_mode_info_block));
+    vesa_data.mode = (struct vbe_mode_info_block *)(vesa_data.info + 1);
     vesa_r.eax = 0x4f00;
     vesa_r.es = (unsigned int)vesa_data.info >> 4;
     vesa_r.edi = 0;
@@ -411,11 +382,6 @@ printf("%x\n",address    );
     
     if ((vesa_r.eax & 0xffff) != 0x4f || strncmp(vesa_data.info->vbe_signature, "VESA", 4) != 0) {
        	fprintf(stderr,"No VESA bios detected!\n");
-        fprintf(stderr,"%04x %i %i %i %i\n",vesa_r.eax,
-        vesa_data.info->vbe_signature[0],
-        vesa_data.info->vbe_signature[1],
-        vesa_data.info->vbe_signature[2],
-        vesa_data.info->vbe_signature[3]);
         fprintf(stderr,"Try running vga_reset.\n");
        	return 1;
     }
@@ -563,18 +529,19 @@ printf("%x\n",address    );
     vesa_r.ebx=0;
     __svgalib_LRMI_int(0x10,&vesa_r);
     vesa_regs_size=vesa_r.ebx*64;
+    __svgalib_LRMI_free_real(vesa_data.info);
 
     SVGALIB_VESA[TEXT]=3;
 
     cardspecs = malloc(sizeof(CardSpecs));
     cardspecs->videoMemory = vesa_memory;
-    cardspecs->maxPixelClock4bpp = 300000;	
-    cardspecs->maxPixelClock8bpp = 300000;	
-    cardspecs->maxPixelClock16bpp = 300000;	
-    cardspecs->maxPixelClock24bpp = 300000;
-    cardspecs->maxPixelClock32bpp = 300000;
+    cardspecs->maxPixelClock4bpp = 75000;	
+    cardspecs->maxPixelClock8bpp = 175500;	
+    cardspecs->maxPixelClock16bpp = 144000;	
+    cardspecs->maxPixelClock24bpp = 75000;
+    cardspecs->maxPixelClock32bpp = 94500;
     cardspecs->flags = CLOCK_PROGRAMMABLE;
-    cardspecs->maxHorizontalCrtc = 4088;
+    cardspecs->maxHorizontalCrtc = 2040;
     cardspecs->nClocks =1;
     cardspecs->clocks = NULL;
     cardspecs->mapClock = vesa_map_clock;
@@ -583,6 +550,8 @@ printf("%x\n",address    );
     __svgalib_driverspecs = &__svgalib_vesa_driverspecs;
 
     LRMI_mem1 = __svgalib_LRMI_alloc_real(vesa_regs_size);
+    LRMI_mem2 = __svgalib_LRMI_alloc_real(sizeof(struct vbe_info_block)
+                                    + sizeof(struct vbe_mode_info_block));
 
     __svgalib_banked_mem_base=0xa0000;
     __svgalib_banked_mem_size=0x10000;
