@@ -80,8 +80,6 @@ unsigned char *__svga_graph_mem_check;
 static int __svgalib_oktowrite = 1;
 /* __svgalib_oktowrite tells if it is safe to write registers. */
 
-static int __svgalib_virtual_mem_fd = -1;
-static int __svga_processnumber = -1;
 #endif
 
 static int __vt_switching_state=RELEASE;
@@ -118,6 +116,13 @@ static void __releasevt_signal(int n)
     printf("Release request.\n");
 #endif
 
+if(release_acquire){
+   fprintf(stderr,"Error at background Release.\n");
+   fprintf(stderr,"This should not happen. Please tell matan@svgalib.org about it\n");
+   exit(3);
+};
+release_acquire=!release_acquire;
+
     forbidvtacquire = 1;
     /*SETSIG(siga, SVGALIB_RELEASE_SIG, releasevt_signal); */
 
@@ -192,6 +197,12 @@ static void __acquirevt_signal(int n)
 #ifdef DEBUG
     printf("Acquisition request.\n");
 #endif
+if(!release_acquire){
+   fprintf(stderr,"Error at background Acquire.\n");
+   fprintf(stderr,"This should not happen. Please tell matan@svgalib.org about it\n");
+   exit(3);
+};
+release_acquire=!release_acquire;
     /* vt_check_last_signal=ACQUIRE; */
     forbidvtrelease = 1;
     /*SETSIG(siga, SVGALIB_ACQUIRE_SIG, acquirevt_signal); */
