@@ -254,22 +254,16 @@ int vga_drawscansegment(unsigned char *colors, int x, int y, int length)
 #endif
 
 	    /* disable Set/Reset Register */
-	    port_out(0x01, GRA_I);
-	    port_out(0x00, GRA_D);
+            __svgalib_outgra(0x01,0x00);
 
 	    /* write to all bits */
-	    port_out(0x08, GRA_I);
-	    port_out(0xFF, GRA_D);
+            __svgalib_outgra(0x08,0xff);
 
 	    /* select write map mask register */
-	    port_out(0x02, SEQ_I);
-	    /* write plane 0 */
-	    port_out(0x01, SEQ_D);
+            __svgalib_outseq(0x02,0x01);
 
 	    /* select read map mask register */
-	    port_out(0x04, GRA_I);
-	    /* read plane 0 */
-	    port_out(0x00, GRA_D);
+            __svgalib_outgra(0x04,0x00);
 	    if (soffs)
 		plane0[0] |= *address & ~mask[soffs];
 	    if (eoffs && l2 == 0)
@@ -277,11 +271,9 @@ int vga_drawscansegment(unsigned char *colors, int x, int y, int length)
 	    MEMCPY(address, plane0, l1);
 
 	    /* write plane 1 */
-	    port_out(0x02, SEQ_I);	/* ATI needs resetting that one */
-	    port_out(0x02, SEQ_D);
+            __svgalib_outseq(0x02,0x02);
 	    /* read plane 1 */
-	    port_out(0x04, GRA_I);	/* ATI needs resetting that one */
-	    port_out(0x01, GRA_D);
+            __svgalib_outgra(0x04,0x01);
 	    if (soffs)
 		plane1[0] |= *address & ~mask[soffs];
 	    if (eoffs && l2 == 0)
@@ -289,11 +281,9 @@ int vga_drawscansegment(unsigned char *colors, int x, int y, int length)
 	    MEMCPY(address, plane1, l1);
 
 	    /* write plane 2 */
-	    port_out(0x02, SEQ_I);	/* ATI needs resetting that one */
-	    port_out(0x04, SEQ_D);
+            __svgalib_outseq(0x02,0x04);
 	    /* read plane 2 */
-	    port_out(0x04, GRA_I);	/* ATI needs resetting that one */
-	    port_out(0x02, GRA_D);
+            __svgalib_outgra(0x04,0x02);
 	    if (soffs)
 		plane2[0] |= *address & ~mask[soffs];
 	    if (eoffs && l2 == 0)
@@ -301,11 +291,9 @@ int vga_drawscansegment(unsigned char *colors, int x, int y, int length)
 	    MEMCPY(address, plane2, l1);
 
 	    /* write plane 3 */
-	    port_out(0x02, SEQ_I);	/* ATI needs resetting that one */
-	    port_out(0x08, SEQ_D);
+            __svgalib_outseq(0x02,0x08);
 	    /* read plane 3 */
-	    port_out(0x04, GRA_I);	/* ATI needs resetting that one */
-	    port_out(0x03, GRA_D);
+            __svgalib_outgra(0x04,0x03);
 	    if (soffs)
 		plane3[0] |= *address & ~mask[soffs];
 	    if (eoffs && l2 == 0)
@@ -316,71 +304,58 @@ int vga_drawscansegment(unsigned char *colors, int x, int y, int length)
 		vga_setpage(page + 1);
 
 		/* write plane 0 */
-		port_out(0x02, SEQ_I);	/* ATI needs resetting that one */
-		port_out(0x01, SEQ_D);
-		if (eoffs) {
+                __svgalib_outseq(0x02,0x01);
+ 		if (eoffs) {
 		    /* read plane 0 */
-		    port_out(0x04, GRA_I);	/* ATI needs resetting that one */
-		    port_out(0x00, GRA_D);
+                    __svgalib_outgra(0x04,0x00);
 		    plane0[k] |= *(GM + l2 - 1) & mask[eoffs];
 		}
 		MEMCPY(GM, &plane0[l1], l2);
 
 		/* write plane 1 */
-		port_out(0x02, SEQ_I);	/* ATI needs resetting that one */
-		port_out(0x02, SEQ_D);
+                __svgalib_outseq(0x02,0x02);
 		if (eoffs) {
 		    /* read plane 1 */
-		    port_out(0x04, GRA_I);	/* ATI needs resetting that one */
-		    port_out(0x01, GRA_D);
+                    __svgalib_outgra(0x04,0x01);
 		    plane1[k] |= *(GM + l2 - 1) & mask[eoffs];
 		}
 		MEMCPY(GM, &plane1[l1], l2);
 
 		/* write plane 2 */
-		port_out(0x02, SEQ_I);	/* ATI needs resetting that one */
-		port_out(0x04, SEQ_D);
+                __svgalib_outseq(0x02,0x04);
 		if (eoffs) {
 		    /* read plane 2 */
-		    port_out(0x04, GRA_I);	/* ATI needs resetting that one */
-		    port_out(0x02, GRA_D);
+                    __svgalib_outgra(0x04,0x02);
 		    plane2[k] |= *(GM + l2 - 1) & mask[eoffs];
 		}
 		MEMCPY(GM, &plane2[l1], l2);
 
 		/* write plane 3 */
-		port_out(0x02, SEQ_I);	/* ATI needs resetting that one */
-		port_out(0x08, SEQ_D);
+                __svgalib_outseq(0x02,0x08);
 		if (eoffs) {
 		    /* read plane 3 */
-		    port_out(0x04, GRA_I);	/* ATI needs resetting that one */
-		    port_out(0x03, GRA_D);
+                    __svgalib_outgra(0x04,0x03);
 		    plane3[k] |= *(GM + l2 - 1) & mask[eoffs];
 		}
 		MEMCPY(GM, &plane3[l1], l2);
 	    }
 	    /* restore map mask register */
-	    port_out(0x02, SEQ_I);	/* ATI needs resetting that one */
-	    port_out(0x0F, SEQ_D);
+            __svgalib_outseq(0x02,0x0f);
 
 	    /* enable Set/Reset Register */
-	    port_out(0x01, GRA_I);
-	    port_out(0x0F, GRA_D);
+            __svgalib_outgra(0x01,0x0f);
 	}
 	break;
     case 2:
 	{
 	    /* disable Set/Reset Register */
-	    port_out(0x01, GRA_I);
-	    port_out(0x00, GRA_D);
+            __svgalib_outgra(0x01,0x00);
 
 	    /* write to all bits */
-	    port_out(0x08, GRA_I);
-	    port_out(0xFF, GRA_D);
+            __svgalib_outgra(0x08,0xff);
 
 	    /* write to all planes */
-	    port_out(0x02, SEQ_I);
-	    port_out(0x0F, SEQ_D);
+            __svgalib_outseq(0x02,0x0f);
 
 #if defined(CONFIG_ALPHA_JENSEN)
 	    MEMCPY(SM + ((y * CI.xdim + x) << 4), colors, length);
@@ -389,12 +364,10 @@ int vga_drawscansegment(unsigned char *colors, int x, int y, int length)
 #endif
 
 	    /* restore map mask register */
-	    port_out(0x02, SEQ_I);	/* Yeah, ATI */
-	    port_out(0x0F, SEQ_D);
+            __svgalib_outseq(0x02,0x0f);
 
 	    /* enable Set/Reset Register */
-	    port_out(0x01, GRA_I);
-	    port_out(0x0F, GRA_D);
+            __svgalib_outgra(0x01,0x0f);
 	}
 	break;
     case 256:
@@ -419,8 +392,7 @@ int vga_drawscansegment(unsigned char *colors, int x, int y, int length)
 
 		    for (plane = 0; plane < 4; plane++) {
 			/* select plane */
-			port_out(0x02, SEQ_I);	/* ATI needs to set it */
-			port_out(1 << plane, SEQ_D);
+                        __svgalib_outseq(0x02,1 << plane );
 
 			pixel = ((4 - (x & 3) + plane) & 3);
 			first = (y * CI.xdim + x) / 4;
@@ -617,46 +589,36 @@ int vga_getscansegment(unsigned char *colors, int x, int y, int length)
 	    l2 = nbytes - l1;
 	    address = GM + offset;
 	    /* disable Set/Reset Register */
-	    port_out(0x01, GRA_I);
-	    port_out(0x00, GRA_D);
+            __svgalib_outgra(0x01,0x00);
 	    /* read plane 0 */
-	    port_out(0x04, GRA_I);
-	    port_out(0x00, GRA_D);
+            __svgalib_outgra(0x04,0x00);
 	    memcpy(plane0, address, l1);
 	    /* read plane 1 */
-	    port_out(0x04, GRA_I);
-	    port_out(0x01, GRA_D);
+            __svgalib_outgra(0x04,0x01);
 	    memcpy(plane1, address, l1);
 	    /* read plane 2 */
-	    port_out(0x04, GRA_I);
-	    port_out(0x02, GRA_D);
+            __svgalib_outgra(0x04,0x02);
 	    memcpy(plane2, address, l1);
 	    /* read plane 3 */
-	    port_out(0x04, GRA_I);
-	    port_out(0x03, GRA_D);
+            __svgalib_outgra(0x04,0x03);
 	    memcpy(plane3, address, l1);
 	    if (l2 > 0) {
 		vga_setpage(page + 1);
 		/* read plane 0 */
-		port_out(0x04, GRA_I);
-		port_out(0x00, GRA_D);
-		memcpy(&plane0[l1], GM, l2);
+              __svgalib_outgra(0x04,0x00);
+ 		memcpy(&plane0[l1], GM, l2);
 		/* read plane 1 */
-		port_out(0x04, GRA_I);
-		port_out(0x01, GRA_D);
+              __svgalib_outgra(0x04,0x01);
 		memcpy(&plane1[l1], GM, l2);
 		/* read plane 2 */
-		port_out(0x04, GRA_I);
-		port_out(0x02, GRA_D);
+              __svgalib_outgra(0x04,0x02);
 		memcpy(&plane2[l1], GM, l2);
 		/* read plane 3 */
-		port_out(0x04, GRA_I);
-		port_out(0x03, GRA_D);
+              __svgalib_outgra(0x04,0x03);
 		memcpy(&plane3[l1], GM, l2);
 	    }
 	    /* enable Set/Reset Register */
-	    port_out(0x01, GRA_I);
-	    port_out(0x0F, GRA_D);
+            __svgalib_outgra(0x01,0x0f);
 	    k = 0;
 	    for (i = 0; i < length;) {
 		for (bit = 7 - soffs; bit >= 0 && i < length; bit--, i++) {
@@ -674,15 +636,12 @@ int vga_getscansegment(unsigned char *colors, int x, int y, int length)
     case 2:
 	{
 	    /* disable Set/Reset Register */
-	    port_out(0x01, GRA_I);
-	    port_out(0x00, GRA_D);
+            __svgalib_outgra(0x01,0x00);
 	    /* read from plane 0 */
-	    port_out(0x04, SEQ_I);
-	    port_out(0x00, SEQ_D);
+            __svgalib_outseq(0x04,0x00);
 	    memcpy(colors, GM + (y * CI.xdim + x) / 8, length);
 	    /* enable Set/Reset Register */
-	    port_out(0x01, GRA_I);
-	    port_out(0x0F, GRA_D);
+            __svgalib_outgra(0x01,0x0f);
 	}
 	break;
     case 256:
@@ -702,8 +661,7 @@ int vga_getscansegment(unsigned char *colors, int x, int y, int length)
 		    int first, offset, pixel, plane;
 		    for (plane = 0; plane < 4; plane++) {
 			/* select plane */
-			port_out(0x04, GRA_I);	/* feel the wrath of the ATI */
-			port_out(plane, GRA_D);
+                        __svgalib_outgra(0x04, plane);
 			pixel = ((4 - (x & 3) + plane) & 3);
 			first = (y * CI.xdim + x) / 4;
 			if((x & 3) + pixel > 3)
