@@ -225,27 +225,23 @@ static void setdisplaystart(int address)
     if (modeinfo->bytesperpixel == 0)	/* not 320x200x256 linear */
 	switch (modeinfo->colors) {
 	case 16:		/* planar 16-color mode */
-	    inb(0x3da);
-	    outb(ATT_IW, 0x13 + 0x20);
-	    outb(ATT_IW, (inb(ATT_R) & 0xf0) | (address & 7));
+            __svgalib_outatt(0x33,(__svgalib_inatt(0x33)&0xf0) | (address & 7));
 	    /* write sa0-2 to bits 0-2 */
 	    address >>= 3;
 	    break;
 	case 256:		/* planar 256-color mode */
-	    inb(0x3da);
-	    outb(ATT_IW, 0x13 + 0x20);
-	    outb(ATT_IW, (inb(ATT_R) & 0xf0) | ((address & 3) << 1));
 	    /* write sa0-1 to bits 1-2 */
+            __svgalib_outatt(0x33,(__svgalib_inatt(0x33)&0xf0) | ((address & 3)<<1) );
 	    address >>= 2;
 	    break;
 	}
-    outw(CRT_IC, 0x0d + (address & 0x00ff) * 256);	/* sa0-sa7 */
-    outw(CRT_IC, 0x0c + (address & 0xff00));	/* sa8-sa15 */
+    __svgalib_outcrtc(0x0d, address & 0x00ff);
+    __svgalib_outcrtc(0x0c, (address & 0xff00) >> 8);
 }
 
 static void setlogicalwidth(int width)
 {
-    outw(CRT_IC, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
+    __svgalib_outcrtc(0x13, width >> 3);
 }
 
 static int vgadrv_init(int, int, int);
