@@ -128,7 +128,7 @@ void __svgalib_setup_VGA_registers(unsigned char *moderegs, ModeTiming * modetim
  * code smaller and inserting small delays between I/O accesses when
  * doing mode switches.
  */
-
+#if 0
 void __svgalib_outGR(int index, unsigned char val)
 {
     int v;
@@ -185,13 +185,4 @@ unsigned char __svgalib_inSR(int index)
     outb(SEQ_I, index);
     return inb(SEQ_D);
 }
-
-int __svgalib_inmisc(void)
-{
-   return inb(MIS_R);
-}
-
-void __svgalib_outmisc(int i)
-{
-   outb(MIS_W,i);
-}
+#endif
